@@ -1,5 +1,6 @@
 import abc
 from dataclasses import dataclass
+from typing import Optional
 
 from yarl import URL
 
@@ -158,6 +159,9 @@ class ImageProgressSave:
 class ImageProgressStep:
     message: str
     layer_id: str
+    status: str
+    current: Optional[float]
+    total: Optional[float]
 
 
 @dataclass(frozen=True)
