@@ -4,6 +4,7 @@ from typing import Any, Dict, List, Mapping, Optional
 
 from neuromation.api.config import Config
 from neuromation.api.core import _Core
+from neuromation.api.server_cfg import Preset
 from neuromation.api.utils import NoPublicConstructor
 
 
@@ -25,6 +26,7 @@ class _ClusterUser:
 
 @dataclass(frozen=True)
 class _Quota:
+    total_running_jobs: Optional[int]
     total_gpu_run_time_minutes: Optional[int]
     total_non_gpu_run_time_minutes: Optional[int]
 
@@ -41,6 +43,8 @@ class _NodePool:
     machine_type: str
     available_cpu: float
     available_memory_mb: int
+    disk_size_gb: int
+    disk_type: Optional[str] = None
     gpu: int = 0
     gpu_model: Optional[str] = None
     is_tpu_enabled: bool = False
@@ -98,6 +102,17 @@ class _Admin(metaclass=NoPublicConstructor):
         async with self._core.request("PUT", url, auth=auth, json=config) as resp:
             resp
 
+    async def update_cluster_resource_presets(
+        self, name: str, presets: Mapping[str, Preset]
+    ) -> None:
+        url = self._config.api_url / "clusters" / name / "orchestrator/resource_presets"
+        auth = await self._config._api_auth()
+        payload = [
+            _serialize_resource_preset(name, preset) for name, preset in presets.items()
+        ]
+        async with self._core.request("PUT", url, auth=auth, json=payload):
+            pass
+
     async def list_cluster_users(
         self, cluster_name: Optional[str] = None
     ) -> List[_ClusterUser]:
@@ -131,8 +146,9 @@ class _Admin(metaclass=NoPublicConstructor):
         self,
         cluster_name: str,
         user_name: str,
-        gpu_value_minutes: Optional[float],
-        non_gpu_value_minutes: Optional[float],
+        total_running_jobs: Optional[int],
+        gpu_value_minutes: Optional[int],
+        non_gpu_value_minutes: Optional[int],
     ) -> _ClusterUserWithQuota:
         url = (
             self._config.admin_url
@@ -144,6 +160,7 @@ class _Admin(metaclass=NoPublicConstructor):
         )
         payload = {
             "quota": {
+                "total_running_jobs": total_running_jobs,
                 "total_gpu_run_time_minutes": gpu_value_minutes,
                 "total_non_gpu_run_time_minutes": non_gpu_value_minutes,
             },
@@ -209,6 +226,7 @@ def _cluster_user_with_quota_from_api(
         user_name=user_name,
         role=_ClusterUserRoleType(payload["role"]),
         quota=_Quota(
+            quota_dict.get("total_running_jobs"),
             quota_dict.get("total_gpu_run_time_minutes"),
             quota_dict.get("total_non_gpu_run_time_minutes"),
         ),
@@ -251,6 +269,8 @@ def _node_pool_from_api(payload: Dict[str, Any]) -> _NodePool:
         machine_type=payload["machine_type"],
         available_cpu=payload["available_cpu"],
         available_memory_mb=payload["available_memory_mb"],
+        disk_type=payload.get("disk_type"),
+        disk_size_gb=payload["disk_size_gb"],
         gpu=payload.get("gpu", 0),
         gpu_model=payload.get("gpu_model"),
         is_tpu_enabled=payload.get("is_tpu_enabled", False),
@@ -260,3 +280,22 @@ def _node_pool_from_api(payload: Dict[str, Any]) -> _NodePool:
 
 def _storage_from_api(payload: Dict[str, Any]) -> _Storage:
     return _Storage(description=payload["description"])
+
+
+def _serialize_resource_preset(name: str, preset: Preset) -> Dict[str, Any]:
+    result: Dict[str, Any] = {
+        "name": name,
+        "cpu": preset.cpu,
+        "memory_mb": preset.memory_mb,
+        "is_preemptible": preset.is_preemptible,
+        "is_preemptible_node_required": preset.is_preemptible_node_required,
+    }
+    if preset.gpu:
+        result["gpu"] = preset.gpu
+        result["gpu_model"] = preset.gpu_model
+    if preset.tpu_type and preset.tpu_software_version:
+        result["tpu"] = {
+            "type": preset.tpu_type,
+            "software_version": preset.tpu_software_version,
+        }
+    return result
