@@ -16,6 +16,7 @@ from typing import (
     Mapping,
     Optional,
     Sequence,
+    Set,
     Union,
 )
 
@@ -43,13 +44,18 @@ from .images import (
 )
 from .parser import DiskVolume, Parser, SecretFile, Volume
 from .parsing_utils import LocalImage, RemoteImage, _as_repo_str, _is_in_neuro_registry
-from .server_cfg import Preset
 from .url_utils import (
     normalize_disk_uri,
     normalize_secret_uri,
     normalize_storage_path_uri,
 )
-from .utils import NoPublicConstructor, asynccontextmanager
+from .utils import NoPublicConstructor
+
+
+if sys.version_info >= (3, 7):  # pragma: no cover
+    from contextlib import asynccontextmanager
+else:
+    from async_generator import asynccontextmanager
 
 
 log = logging.getLogger(__name__)
@@ -74,6 +80,7 @@ class JobStatus(str, enum.Enum):
     PENDING: a job is being created and scheduled. This includes finding (and
     possibly waiting for) sufficient amount of resources, pulling an image
     from a registry etc.
+    SUSPENDED: a preemptible job is paused to allow other jobs to run.
     RUNNING: a job is being run.
     SUCCEEDED: a job terminated with the 0 exit code.
     CANCELLED: a running job was manually terminated/deleted.
@@ -81,12 +88,37 @@ class JobStatus(str, enum.Enum):
     """
 
     PENDING = "pending"
+    SUSPENDED = "suspended"
     RUNNING = "running"
     SUCCEEDED = "succeeded"
     FAILED = "failed"
     CANCELLED = "cancelled"
     UNKNOWN = "unknown"  # invalid status code, a default value is status is not sent
 
+    @property
+    def is_pending(self) -> bool:
+        return self in (self.PENDING, self.SUSPENDED)
+
+    @property
+    def is_running(self) -> bool:
+        return self == self.RUNNING
+
+    @property
+    def is_finished(self) -> bool:
+        return self in (self.SUCCEEDED, self.FAILED, self.CANCELLED)
+
+    @classmethod
+    def items(cls) -> Set["JobStatus"]:
+        return {item for item in cls if item != cls.UNKNOWN}
+
+    @classmethod
+    def active_items(cls) -> Set["JobStatus"]:
+        return {item for item in cls.items() if not item.is_finished}
+
+    @classmethod
+    def finished_items(cls) -> Set["JobStatus"]:
+        return {item for item in cls.items() if item.is_finished}
+
 
 @dataclass(frozen=True)
 class HTTPPort:
@@ -115,7 +147,7 @@ class JobStatusHistory:
     status: JobStatus
     reason: str
     description: str
-    restarts: int
+    restarts: int = 0
     created_at: Optional[datetime] = None
     started_at: Optional[datetime] = None
     finished_at: Optional[datetime] = None
@@ -143,6 +175,7 @@ class JobDescription:
     history: JobStatusHistory
     container: Container
     is_preemptible: bool
+    pass_config: bool
     uri: URL
     name: Optional[str] = None
     tags: Sequence[str] = ()
@@ -152,6 +185,9 @@ class JobDescription:
     internal_hostname_named: Optional[str] = None
     restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER
     life_span: Optional[float] = None
+    preset_name: Optional[str] = None
+    is_preemptible_node_required: bool = False
+    privileged: bool = False
 
 
 @dataclass(frozen=True)
@@ -218,28 +254,101 @@ class Jobs(metaclass=NoPublicConstructor):
         tags: Sequence[str] = (),
         description: Optional[str] = None,
         is_preemptible: bool = False,
+        pass_config: bool = False,
+        wait_for_jobs_quota: bool = False,
         schedule_timeout: Optional[float] = None,
         restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER,
         life_span: Optional[float] = None,
     ) -> JobDescription:
         url = self._config.api_url / "jobs"
-        payload: Dict[str, Any] = {
-            "container": _container_to_api(container, self._config),
-            "is_preemptible": is_preemptible,
-        }
-        if name:
-            payload["name"] = name
-        if tags:
-            payload["tags"] = tags
-        if description:
-            payload["description"] = description
-        if schedule_timeout:
-            payload["schedule_timeout"] = schedule_timeout
-        if restart_policy != JobRestartPolicy.NEVER:
-            payload["restart_policy"] = str(restart_policy)
-        if life_span is not None:
-            payload["max_run_time_minutes"] = int(life_span // 60)
-        payload["cluster_name"] = self._config.cluster_name
+        payload = _job_to_api(
+            config=self._config,
+            name=name,
+            tags=tags,
+            description=description,
+            pass_config=pass_config,
+            wait_for_jobs_quota=wait_for_jobs_quota,
+            schedule_timeout=schedule_timeout,
+            restart_policy=restart_policy,
+            life_span=life_span,
+        )
+        payload["container"] = _container_to_api(
+            config=self._config,
+            image=container.image,
+            entrypoint=container.entrypoint,
+            command=container.command,
+            working_dir=container.working_dir,
+            http=container.http,
+            env=container.env,
+            volumes=container.volumes,
+            secret_env=container.secret_env,
+            secret_files=container.secret_files,
+            disk_volumes=container.disk_volumes,
+            tty=container.tty,
+        )
+        payload["container"]["resources"] = _resources_to_api(container.resources)
+        payload["is_preemptible"] = is_preemptible
+        auth = await self._config._api_auth()
+        async with self._core.request("POST", url, json=payload, auth=auth) as resp:
+            res = await resp.json()
+            return _job_description_from_api(res, self._parse)
+
+    async def start(
+        self,
+        *,
+        image: RemoteImage,
+        preset_name: str,
+        entrypoint: Optional[str] = None,
+        command: Optional[str] = None,
+        working_dir: Optional[str] = None,
+        http: Optional[HTTPPort] = None,
+        env: Optional[Mapping[str, str]] = None,
+        volumes: Sequence[Volume] = (),
+        secret_env: Optional[Mapping[str, URL]] = None,
+        secret_files: Sequence[SecretFile] = (),
+        disk_volumes: Sequence[DiskVolume] = (),
+        tty: bool = False,
+        shm: bool = False,
+        name: Optional[str] = None,
+        tags: Sequence[str] = (),
+        description: Optional[str] = None,
+        pass_config: bool = False,
+        wait_for_jobs_quota: bool = False,
+        schedule_timeout: Optional[float] = None,
+        restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER,
+        life_span: Optional[float] = None,
+        privileged: bool = False,
+    ) -> JobDescription:
+        url = (self._config.api_url / "jobs").with_query("from_preset")
+        container_payload = _container_to_api(
+            config=self._config,
+            image=image,
+            entrypoint=entrypoint,
+            command=command,
+            working_dir=working_dir,
+            http=http,
+            env=env,
+            volumes=volumes,
+            secret_env=secret_env,
+            secret_files=secret_files,
+            disk_volumes=disk_volumes,
+            tty=tty,
+            shm=shm,
+        )
+        payload = _job_to_api(
+            config=self._config,
+            name=name,
+            preset_name=preset_name,
+            tags=tags,
+            description=description,
+            pass_config=pass_config,
+            wait_for_jobs_quota=wait_for_jobs_quota,
+            schedule_timeout=schedule_timeout,
+            restart_policy=restart_policy,
+            life_span=life_span,
+            privileged=privileged,
+        )
+        payload.update(**container_payload)
         auth = await self._config._api_auth()
         async with self._core.request("POST", url, json=payload, auth=auth) as resp:
             res = await resp.json()
@@ -567,19 +676,10 @@ class Jobs(metaclass=NoPublicConstructor):
         async with self._core.request("POST", url, auth=auth) as resp:
             resp
 
-    async def get_capacity(self, presets: Mapping[str, Preset]) -> Mapping[str, int]:
-        url = self._config.monitoring_url / "available"
+    async def get_capacity(self) -> Mapping[str, int]:
+        url = self._config.monitoring_url / "capacity"
         auth = await self._config._api_auth()
-        payload: Dict[str, Any] = {}
-        for name, preset in presets.items():
-            payload[name] = {
-                "cpu": preset.cpu,
-                "memory_mb": preset.memory_mb,
-                "gpu": preset.gpu or 0,
-                "gpu_model": preset.gpu_model or "",
-                "is_preemptible": preset.is_preemptible,
-            }
-        async with self._core.request("POST", url, auth=auth, json=payload) as resp:
+        async with self._core.request("GET", url, auth=auth) as resp:
             return await resp.json()
 
 
@@ -690,37 +790,50 @@ def _container_from_api(data: Dict[str, Any], parse: Parser) -> Container:
     )
 
 
-def _container_to_api(container: Container, config: Config) -> Dict[str, Any]:
-    primitive: Dict[str, Any] = {
-        "image": _as_repo_str(container.image),
-        "resources": _resources_to_api(container.resources),
-    }
-    if container.entrypoint:
-        primitive["entrypoint"] = container.entrypoint
-    if container.command:
-        primitive["command"] = container.command
-    if container.working_dir:
-        primitive["working_dir"] = container.working_dir
-    if container.http:
-        primitive["http"] = _http_port_to_api(container.http)
-    if container.env:
-        primitive["env"] = container.env
-    if container.volumes:
-        primitive["volumes"] = [_volume_to_api(v, config) for v in container.volumes]
-    if container.secret_env:
+def _container_to_api(
+    config: Config,
+    image: RemoteImage,
+    entrypoint: Optional[str] = None,
+    command: Optional[str] = None,
+    working_dir: Optional[str] = None,
+    http: Optional[HTTPPort] = None,
+    env: Optional[Mapping[str, str]] = None,
+    volumes: Sequence[Volume] = (),
+    secret_env: Optional[Mapping[str, URL]] = None,
+    secret_files: Sequence[SecretFile] = (),
+    disk_volumes: Sequence[DiskVolume] = (),
+    tty: bool = False,
+    shm: bool = False,
+) -> Dict[str, Any]:
+    primitive: Dict[str, Any] = {"image": _as_repo_str(image)}
+    if shm:
+        primitive["resources"] = {"shm": shm}
+    if entrypoint:
+        primitive["entrypoint"] = entrypoint
+    if command:
+        primitive["command"] = command
+    if working_dir:
+        primitive["working_dir"] = working_dir
+    if http:
+        primitive["http"] = _http_port_to_api(http)
+    if env:
+        primitive["env"] = env
+    if volumes:
+        primitive["volumes"] = [_volume_to_api(v, config) for v in volumes]
+    if secret_env:
         primitive["secret_env"] = {
             k: str(normalize_secret_uri(v, config.username, config.cluster_name))
-            for k, v in container.secret_env.items()
+            for k, v in secret_env.items()
         }
-    if container.secret_files:
+    if secret_files:
         primitive["secret_volumes"] = [
-            _secret_file_to_api(v, config) for v in container.secret_files
+            _secret_file_to_api(v, config) for v in secret_files
         ]
-    if container.disk_volumes:
+    if disk_volumes:
         primitive["disk_volumes"] = [
-            _disk_volume_to_api(v, config) for v in container.disk_volumes
+            _disk_volume_to_api(v, config) for v in disk_volumes
         ]
-    if container.tty:
+    if tty:
         primitive["tty"] = True
     return primitive
 
@@ -768,6 +881,8 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
         history=history,
         container=container,
         is_preemptible=res["is_preemptible"],
+        is_preemptible_node_required=res.get("is_preemptible_node_required", False),
+        pass_config=res["pass_config"],
         name=name,
         tags=tags,
         description=description,
@@ -777,9 +892,46 @@ def _job_description_from_api(res: Dict[str, Any], parse: Parser) -> JobDescript
         uri=URL(res["uri"]),
         restart_policy=restart_policy,
         life_span=life_span,
+        preset_name=res.get("preset_name"),
     )
 
 
+def _job_to_api(
+    config: Config,
+    name: Optional[str] = None,
+    preset_name: Optional[str] = None,
+    tags: Sequence[str] = (),
+    description: Optional[str] = None,
+    pass_config: bool = False,
+    wait_for_jobs_quota: bool = False,
+    schedule_timeout: Optional[float] = None,
+    restart_policy: JobRestartPolicy = JobRestartPolicy.NEVER,
+    life_span: Optional[float] = None,
+    privileged: bool = False,
+) -> Dict[str, Any]:
+    primitive: Dict[str, Any] = {"pass_config": pass_config}
+    if name:
+        primitive["name"] = name
+    if preset_name:
+        primitive["preset_name"] = preset_name
+    if tags:
+        primitive["tags"] = tags
+    if description:
+        primitive["description"] = description
+    if schedule_timeout:
+        primitive["schedule_timeout"] = schedule_timeout
+    if restart_policy != JobRestartPolicy.NEVER:
+        primitive["restart_policy"] = str(restart_policy)
+    if life_span is not None:
+        primitive["max_run_time_minutes"] = int(life_span // 60)
+    if wait_for_jobs_quota:
+        primitive["wait_for_jobs_quota"] = wait_for_jobs_quota
+    if privileged:
+        primitive["privileged"] = privileged
+    primitive["cluster_name"] = config.cluster_name
+    return primitive
+
+
 def _job_telemetry_from_api(value: Dict[str, Any]) -> JobTelemetry:
     return JobTelemetry(
         cpu=value["cpu"],
