@@ -13,6 +13,7 @@ class Preset:
     cpu: float
     memory_mb: int
     is_preemptible: bool = False
+    is_preemptible_node_required: bool = False
     gpu: Optional[int] = None
     gpu_model: Optional[str] = None
     tpu_type: Optional[str] = None
@@ -50,6 +51,9 @@ def _parse_cluster_config(payload: Dict[str, Any]) -> Cluster:
             gpu=data.get("gpu"),
             gpu_model=data.get("gpu_model"),
             is_preemptible=data.get("is_preemptible", False),
+            is_preemptible_node_required=data.get(
+                "is_preemptible_node_required", False
+            ),
             tpu_type=tpu_type,
             tpu_software_version=tpu_software_version,
         )
