@@ -1,5 +1,4 @@
 import asyncio
-import datetime
 import enum
 import errno
 import fnmatch
@@ -9,7 +8,7 @@ import os
 import re
 import time
 from dataclasses import dataclass
-from email.utils import parsedate
+from email.utils import parsedate_to_datetime
 from pathlib import Path
 from stat import S_ISREG
 from typing import (
@@ -31,8 +30,6 @@ import aiohttp
 import attr
 from yarl import URL
 
-from neuromation.api.utils import QueuedCall, queue_calls
-
 from .abc import (
     AbstractDeleteProgress,
     AbstractFileProgress,
@@ -58,7 +55,7 @@ from .url_utils import (
     normalize_storage_path_uri,
 )
 from .users import Action
-from .utils import NoPublicConstructor, retries
+from .utils import NoPublicConstructor, QueuedCall, queue_calls, retries
 
 
 log = logging.getLogger(__name__)
@@ -121,35 +118,55 @@ class Storage(metaclass=NoPublicConstructor):
 
     def _set_time_diff(self, request_time: float, resp: aiohttp.ClientResponse) -> None:
         response_time = time.time()
-        str_date = resp.headers.get("Date")
-        if not str_date:
+        try:
+            server_dt = parsedate_to_datetime(resp.headers.get("Date", ""))
+        except ValueError:
             return
-        server_timetuple = parsedate(str_date)
-        if not server_timetuple:
-            return
-        server_time = datetime.datetime(
-            *server_timetuple[:6], tzinfo=datetime.timezone.utc
-        ).timestamp()
+        server_time = server_dt.timestamp()
         # Remove 1 because server time has been truncated
-        # and can be up to 1 second less than the actulal value
+        # and can be up to 1 second less than the actual value.
         self._min_time_diff = request_time - server_time - 1.0
         self._max_time_diff = response_time - server_time
 
-    def _is_local_modified(self, local: os.stat_result, remote: FileStatus) -> bool:
-        return (
-            local.st_size != remote.size
-            or local.st_mtime - remote.modification_time
+    def _check_upload(
+        self, local: os.stat_result, remote: FileStatus, update: bool, continue_: bool
+    ) -> Optional[int]:
+        if (
+            local.st_mtime - remote.modification_time
             > self._min_time_diff - TIME_THRESHOLD
-        )
+        ):
+            # Local is likely newer.
+            return 0
+        # Remote is definitely newer.
+        if update:
+            return None
+        if continue_:
+            if local.st_size == remote.size:  # complete
+                return None
+            if local.st_size > remote.size:  # partial
+                return remote.size
+        return 0
 
-    def _is_remote_modified(self, local: os.stat_result, remote: FileStatus) -> bool:
+    def _check_download(
+        self, local: os.stat_result, remote: FileStatus, update: bool, continue_: bool
+    ) -> Optional[int]:
         # Add 1 because remote.modification_time has been truncated
-        # and can be up to 1 second less than the actulal value
-        return (
-            local.st_size != remote.size
-            or local.st_mtime - remote.modification_time
+        # and can be up to 1 second less than the actual value.
+        if (
+            local.st_mtime - remote.modification_time
             < self._max_time_diff + TIME_THRESHOLD + 1.0
-        )
+        ):
+            # Remote is likely newer.
+            return 0
+        # Local is definitely newer.
+        if update:
+            return None
+        if continue_:
+            if local.st_size == remote.size:  # complete
+                return None
+            if local.st_size < remote.size:  # partial
+                return local.st_size
+        return 0
 
     async def ls(self, uri: URL) -> AsyncIterator[FileStatus]:
         uri = self._normalize_uri(uri)
@@ -277,6 +294,21 @@ class Storage(metaclass=NoPublicConstructor):
         ) as resp:
             resp  # resp.status == 201
 
+    async def write(self, uri: URL, data: bytes, offset: int) -> None:
+        if not data:
+            raise ValueError("empty data")
+        path = self._uri_to_path(uri)
+        url = self._config.storage_url / path
+        url = url.with_query(op="WRITE")
+        timeout = attr.evolve(self._core.timeout, sock_read=None)
+        auth = await self._config._api_auth()
+        headers = {"Content-Range": f"bytes {offset}-{offset + len(data) - 1}/*"}
+
+        async with self._core.request(
+            "PATCH", url, data=data, timeout=timeout, auth=auth, headers=headers
+        ) as resp:
+            resp  # resp.status == 200
+
     async def stat(self, uri: URL) -> FileStatus:
         uri = self._normalize_uri(uri)
         url = self._config.storage_url / self._uri_to_path(uri, normalized=True)
@@ -293,13 +325,40 @@ class Storage(metaclass=NoPublicConstructor):
                 self._config.cluster_name, res["FileStatus"]
             )
 
-    async def open(self, uri: URL) -> AsyncIterator[bytes]:
+    async def open(
+        self, uri: URL, offset: int = 0, size: Optional[int] = None
+    ) -> AsyncIterator[bytes]:
         url = self._config.storage_url / self._uri_to_path(uri)
         url = url.with_query(op="OPEN")
         timeout = attr.evolve(self._core.timeout, sock_read=None)
         auth = await self._config._api_auth()
+        if offset < 0:
+            raise ValueError("offset should be >= 0")
+        if size is None:
+            if offset:
+                partial = True
+                headers = {"Range": f"bytes={offset}-"}
+            else:
+                partial = False
+                headers = {}
+        elif size > 0:
+            partial = True
+            headers = {"Range": f"bytes={offset}-{offset + size - 1}"}
+        elif not size:
+            return
+        else:
+            raise ValueError("size should be >= 0")
+
+        async with self._core.request(
+            "GET", url, timeout=timeout, auth=auth, headers=headers
+        ) as resp:
+            if partial:
+                if resp.status != aiohttp.web.HTTPPartialContent.status_code:
+                    raise RuntimeError(f"Unexpected status code {resp.status}")
+                rng = _parse_content_range(resp.headers.get("Content-Range"))
+                if rng.start != offset:
+                    raise RuntimeError("Invalid header Content-Range")
 
-        async with self._core.request("GET", url, timeout=timeout, auth=auth) as resp:
             async for data in resp.content.iter_any():
                 yield data
 
@@ -374,34 +433,13 @@ class Storage(metaclass=NoPublicConstructor):
 
     # high-level helpers
 
-    async def _iterate_file(
-        self,
-        src: Path,
-        dst: URL,
-        *,
-        progress: _AsyncAbstractFileProgress,
-    ) -> AsyncIterator[bytes]:
-        loop = asyncio.get_event_loop()
-        src_url = URL(src.as_uri())
-        async with self._file_sem:
-            with src.open("rb") as stream:
-                size = os.stat(stream.fileno()).st_size
-                await progress.start(StorageProgressStart(src_url, dst, size))
-                chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
-                pos = len(chunk)
-                while chunk:
-                    await progress.step(StorageProgressStep(src_url, dst, pos, size))
-                    yield chunk
-                    chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
-                    pos += len(chunk)
-                await progress.complete(StorageProgressComplete(src_url, dst, size))
-
     async def upload_file(
         self,
         src: URL,
         dst: URL,
         *,
         update: bool = False,
+        continue_: bool = False,
         progress: Optional[AbstractFileProgress] = None,
     ) -> None:
         src = normalize_local_path_uri(src)
@@ -421,6 +459,7 @@ class Storage(metaclass=NoPublicConstructor):
                 raise
             # Ignore stat errors for device files like NUL or CON on Windows.
             # See https://bugs.python.org/issue37074
+        offset: Optional[int] = 0
         try:
             dst_stat = await self.stat(dst)
             if dst_stat.is_dir():
@@ -439,34 +478,61 @@ class Storage(metaclass=NoPublicConstructor):
                     errno.ENOTDIR, "Not a directory", str(dst.parent)
                 )
         else:
-            if update:
+            if update or continue_:
                 try:
                     src_stat = path.stat()
                 except OSError:
                     pass
                 else:
-                    if S_ISREG(src_stat.st_mode) and not self._is_local_modified(
-                        src_stat, dst_stat
-                    ):
-                        return
+                    if S_ISREG(src_stat.st_mode):
+                        offset = self._check_upload(
+                            src_stat, dst_stat, update, continue_
+                        )
+        if offset is None:
+            return
 
         async_progress: _AsyncAbstractFileProgress
         queue, async_progress = queue_calls(progress)
-        await run_progress(queue, self._upload_file(path, dst, progress=async_progress))
+        await run_progress(
+            queue, self._upload_file(path, dst, offset, progress=async_progress)
+        )
 
     async def _upload_file(
         self,
         src_path: Path,
         dst: URL,
+        offset: int,
         *,
         progress: _AsyncAbstractFileProgress,
     ) -> None:
-        for retry in retries(f"Fail to upload {dst}"):
-            async with retry:
-                await self.create(
-                    dst,
-                    self._iterate_file(src_path, dst, progress=progress),
-                )
+        src = URL(src_path.as_uri())
+        loop = asyncio.get_event_loop()
+        async with self._file_sem:
+            with src_path.open("rb") as stream:
+                size = os.stat(stream.fileno()).st_size
+                await progress.start(StorageProgressStart(src, dst, size))
+
+                if offset:
+                    stream.seek(offset)
+                else:
+                    chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
+                    for retry in retries(f"Fail to upload {dst}"):
+                        async with retry:
+                            await self.create(dst, chunk)
+                    offset = len(chunk)
+
+                if offset:
+                    while True:
+                        await progress.step(StorageProgressStep(src, dst, offset, size))
+                        chunk = await loop.run_in_executor(None, stream.read, READ_SIZE)
+                        if not chunk:
+                            break
+                        for retry in retries(f"Fail to upload {dst}"):
+                            async with retry:
+                                await self.write(dst, chunk, offset)
+                        offset += len(chunk)
+
+                await progress.complete(StorageProgressComplete(src, dst, size))
 
     async def upload_dir(
         self,
@@ -474,6 +540,7 @@ class Storage(metaclass=NoPublicConstructor):
         dst: URL,
         *,
         update: bool = False,
+        continue_: bool = False,
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
         ignore_file_names: AbstractSet[str] = frozenset(),
         progress: Optional[AbstractRecursiveFileProgress] = None,
@@ -499,6 +566,7 @@ class Storage(metaclass=NoPublicConstructor):
                 dst,
                 "",
                 update=update,
+                continue_=continue_,
                 filter=filter,
                 ignore_file_names=ignore_file_names,
                 progress=async_progress,
@@ -513,6 +581,7 @@ class Storage(metaclass=NoPublicConstructor):
         rel_path: str,
         *,
         update: bool,
+        continue_: bool,
         filter: Callable[[str], Awaitable[bool]],
         ignore_file_names: AbstractSet[str],
         progress: _AsyncAbstractRecursiveFileProgress,
@@ -520,7 +589,7 @@ class Storage(metaclass=NoPublicConstructor):
         tasks = []
         try:
             exists = False
-            if update:
+            if update or continue_:
                 try:
                     for retry in retries(f"Fail to list {dst}"):
                         async with retry:
@@ -531,7 +600,7 @@ class Storage(metaclass=NoPublicConstructor):
                             }
                     exists = True
                 except ResourceNotFound:
-                    update = False
+                    update = continue_ = False
             if not exists:
                 for retry in retries(f"Fail to create {dst}"):
                     async with retry:
@@ -561,14 +630,17 @@ class Storage(metaclass=NoPublicConstructor):
                 log.debug(f"Skip {child_rel_path}")
                 continue
             if child.is_file():
-                if (
-                    update
-                    and name in dst_files
-                    and not self._is_local_modified(child.stat(), dst_files[name])
-                ):
+                offset: Optional[int] = 0
+                if (update or continue_) and name in dst_files:
+                    offset = self._check_upload(
+                        child.stat(), dst_files[name], update, continue_
+                    )
+                if offset is None:
                     continue
                 tasks.append(
-                    self._upload_file(src_path / name, dst / name, progress=progress)
+                    self._upload_file(
+                        src_path / name, dst / name, offset, progress=progress
+                    )
                 )
             elif child.is_dir():
                 tasks.append(
@@ -578,6 +650,7 @@ class Storage(metaclass=NoPublicConstructor):
                         dst / name,
                         child_rel_path,
                         update=update,
+                        continue_=continue_,
                         filter=filter,
                         ignore_file_names=ignore_file_names,
                         progress=progress,
@@ -603,6 +676,7 @@ class Storage(metaclass=NoPublicConstructor):
         dst: URL,
         *,
         update: bool = False,
+        continue_: bool = False,
         progress: Optional[AbstractFileProgress] = None,
     ) -> None:
         src = normalize_storage_path_uri(
@@ -613,21 +687,25 @@ class Storage(metaclass=NoPublicConstructor):
         src_stat = await self.stat(src)
         if not src_stat.is_file():
             raise IsADirectoryError(errno.EISDIR, "Is a directory", str(src))
-        if update:
+        offset: Optional[int] = 0
+        if update or continue_:
             try:
                 dst_stat = path.stat()
             except OSError:
                 pass
             else:
-                if S_ISREG(dst_stat.st_mode) and not self._is_remote_modified(
-                    dst_stat, src_stat
-                ):
-                    return
+                if S_ISREG(dst_stat.st_mode):
+                    offset = self._check_download(dst_stat, src_stat, update, continue_)
+        if offset is None:
+            return
+
         async_progress: _AsyncAbstractFileProgress
         queue, async_progress = queue_calls(progress)
         await run_progress(
             queue,
-            self._download_file(src, dst, path, src_stat.size, progress=async_progress),
+            self._download_file(
+                src, dst, path, src_stat.size, offset, progress=async_progress
+            ),
         )
 
     async def _download_file(
@@ -636,22 +714,31 @@ class Storage(metaclass=NoPublicConstructor):
         dst: URL,
         dst_path: Path,
         size: int,
+        offset: int,
         *,
         progress: _AsyncAbstractFileProgress,
     ) -> None:
         loop = asyncio.get_event_loop()
         async with self._file_sem:
             await progress.start(StorageProgressStart(src, dst, size))
-            for retry in retries(f"Fail to download {src}"):
-                async with retry:
-                    with dst_path.open("wb") as stream:
-                        pos = 0
-                        async for chunk in self.open(src):
+            with dst_path.open("rb+" if offset else "wb") as stream:
+                if offset:
+                    stream.seek(offset)
+                for retry in retries(f"Fail to download {src}"):
+                    pos = stream.tell()
+                    if pos >= size:
+                        break
+                    async with retry:
+                        it = self.open(src, offset=pos)
+                        async for chunk in it:
                             pos += len(chunk)
                             await progress.step(
                                 StorageProgressStep(src, dst, pos, size)
                             )
                             await loop.run_in_executor(None, stream.write, chunk)
+                            if chunk:
+                                retry.reset()
+
             await progress.complete(StorageProgressComplete(src, dst, size))
 
     async def download_dir(
@@ -660,6 +747,7 @@ class Storage(metaclass=NoPublicConstructor):
         dst: URL,
         *,
         update: bool = False,
+        continue_: bool = False,
         filter: Optional[Callable[[str], Awaitable[bool]]] = None,
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
@@ -681,6 +769,7 @@ class Storage(metaclass=NoPublicConstructor):
                 path,
                 "",
                 update=update,
+                continue_=continue_,
                 filter=filter,
                 progress=async_progress,
             ),
@@ -694,13 +783,14 @@ class Storage(metaclass=NoPublicConstructor):
         rel_path: str,
         *,
         update: bool,
+        continue_: bool,
         filter: Callable[[str], Awaitable[bool]],
         progress: _AsyncAbstractRecursiveFileProgress,
     ) -> None:
         dst_path.mkdir(parents=True, exist_ok=True)
         await progress.enter(StorageProgressEnterDir(src, dst))
         tasks = []
-        if update:
+        if update or continue_:
             loop = asyncio.get_event_loop()
             async with self._file_sem:
                 dst_files = await loop.run_in_executor(
@@ -723,11 +813,12 @@ class Storage(metaclass=NoPublicConstructor):
                 log.debug(f"Skip {child_rel_path}")
                 continue
             if child.is_file():
-                if (
-                    update
-                    and name in dst_files
-                    and not self._is_remote_modified(dst_files[name].stat(), child)
-                ):
+                offset: Optional[int] = 0
+                if (update or continue_) and name in dst_files:
+                    offset = self._check_download(
+                        dst_files[name].stat(), child, update, continue_
+                    )
+                if offset is None:
                     continue
                 tasks.append(
                     self._download_file(
@@ -735,6 +826,7 @@ class Storage(metaclass=NoPublicConstructor):
                         dst / name,
                         dst_path / name,
                         child.size,
+                        offset,
                         progress=progress,
                     )
                 )
@@ -746,6 +838,7 @@ class Storage(metaclass=NoPublicConstructor):
                         dst_path / name,
                         child_rel_path,
                         update=update,
+                        continue_=continue_,
                         filter=filter,
                         progress=progress,
                     )
@@ -800,6 +893,19 @@ def _file_status_from_api_stat(cluster_name: str, values: Dict[str, Any]) -> Fil
     )
 
 
+def _parse_content_range(rng_str: Optional[str]) -> slice:
+    if rng_str is None:
+        raise RuntimeError("Missed header Content-Range")
+    m = re.fullmatch(r"bytes (\d+)-(\d+)/(\d+|\*)", rng_str)
+    if not m:
+        raise RuntimeError("Invalid header Content-Range")
+    start = int(m[1])
+    end = int(m[2])
+    if end < start:
+        raise RuntimeError("Invalid header Content-Range" + rng_str)
+    return slice(start, end + 1)
+
+
 ProgressQueueItem = Optional[Tuple[Callable[[Any], None], Any]]
 
 
