@@ -73,19 +73,28 @@ class QuotaFormatter:
     QUOTA_NOT_SET = "infinity"
 
     def __call__(self, quota: _Quota) -> RenderableType:
+        jobs_details = self._format_quota_details(
+            quota.total_running_jobs, is_minutes=False
+        )
         gpu_details = self._format_quota_details(quota.total_gpu_run_time_minutes)
         non_gpu_details = self._format_quota_details(
             quota.total_non_gpu_run_time_minutes
         )
         return RenderGroup(
-            f"[b]GPU[/b]: {gpu_details}", f"[b]CPU[/b]: {non_gpu_details}"
+            f"[b]Jobs[/b]: {jobs_details}",
+            f"[b]GPU[/b]: {gpu_details}",
+            f"[b]CPU[/b]: {non_gpu_details}",
         )
 
-    def _format_quota_details(self, run_time_minutes: Optional[int]) -> str:
-        if run_time_minutes is None:
+    def _format_quota_details(
+        self, quota: Optional[int], *, is_minutes: bool = True
+    ) -> str:
+        if quota is None:
             return self.QUOTA_NOT_SET
+        elif is_minutes:
+            return f"{quota}m"
         else:
-            return f"{run_time_minutes}m"
+            return str(quota)
 
 
 class ClustersFormatter:
@@ -123,6 +132,7 @@ def _format_presets(
     table.add_column("#CPU", justify="right")
     table.add_column("Memory", justify="right")
     table.add_column("Preemptible", justify="center")
+    table.add_column("Preemptible Node", justify="center")
     table.add_column("GPU", justify="left")
     if available_jobs_counts:
         table.add_column("Jobs Avail", justify="right")
@@ -138,6 +148,7 @@ def _format_presets(
             str(preset.cpu),
             format_size(preset.memory_mb * 1024 ** 2),
             "√" if preset.is_preemptible else "×",
+            "√" if preset.is_preemptible_node_required else "×",
             gpu,
         ]
         if has_tpu:
