@@ -11,18 +11,18 @@ from typing import List, Optional, Sequence, Set, Tuple
 import async_timeout
 import click
 from dateutil.parser import isoparse
+from rich.table import Table
 from yarl import URL
 
 from neuromation.api import (
+    PASS_CONFIG_ENV_NAME,
     AuthorizationError,
     Client,
-    Container,
     HTTPPort,
     JobDescription,
     JobRestartPolicy,
     JobStatus,
     RemoteImage,
-    Resources,
     Volume,
 )
 from neuromation.cli.formatters.images import DockerImageProgress
@@ -38,18 +38,10 @@ from .click_types import (
     JOB_COLUMNS,
     JOB_NAME,
     LOCAL_REMOTE_PORT,
-    MEGABYTE,
     PRESET,
     ImageType,
 )
 from .const import EX_PLATFORMERROR
-from .defaults import (
-    GPU_MODELS,
-    JOB_CPU_NUMBER,
-    JOB_GPU_MODEL,
-    JOB_GPU_NUMBER,
-    JOB_MEMORY_AMOUNT,
-)
 from .formatters.jobs import (
     BaseJobsFormatter,
     JobStartProgress,
@@ -66,7 +58,6 @@ from .parse_utils import (
 )
 from .root import Root
 from .utils import (
-    NEURO_STEAL_CONFIG,
     AsyncExitStack,
     alias,
     argument,
@@ -75,7 +66,6 @@ from .utils import (
     deprecated_quiet_option,
     group,
     option,
-    pager_maybe,
     resolve_job,
     volume_to_verbose_str,
 )
@@ -108,269 +98,6 @@ def job() -> None:
     """
 
 
-@command(
-    context_settings=dict(allow_interspersed_args=False), hidden=True, deprecated=True
-)
-@argument("image", type=ImageType())
-@argument("cmd", nargs=-1, type=click.UNPROCESSED)
-@option(
-    "-g",
-    "--gpu",
-    metavar="NUMBER",
-    type=int,
-    help="Number of GPUs to request",
-    default=JOB_GPU_NUMBER,
-    show_default=True,
-)
-@option(
-    "--gpu-model",
-    metavar="MODEL",
-    type=click.Choice(GPU_MODELS),
-    help="GPU to use",
-    default=JOB_GPU_MODEL,
-    show_default=True,
-)
-@option("--tpu-type", metavar="TYPE", type=str, help="TPU type to use")
-@option(
-    "tpu_software_version",
-    "--tpu-sw-version",
-    metavar="VERSION",
-    type=str,
-    help="Requested TPU software version",
-)
-@option(
-    "-c",
-    "--cpu",
-    metavar="NUMBER",
-    type=float,
-    help="Number of CPUs to request",
-    default=JOB_CPU_NUMBER,
-    show_default=True,
-)
-@option(
-    "-m",
-    "--memory",
-    metavar="AMOUNT",
-    type=MEGABYTE,
-    help="Memory amount to request",
-    default=JOB_MEMORY_AMOUNT,
-    show_default=True,
-)
-@option(
-    "-x/-X",
-    "--extshm/--no-extshm",
-    is_flag=True,
-    default=True,
-    show_default=True,
-    help="Request extended '/dev/shm' space",
-)
-@option("--http", type=int, metavar="PORT", help="Enable HTTP port forwarding to a job")
-@option(
-    "--http-auth/--no-http-auth",
-    is_flag=True,
-    help="Enable HTTP authentication for forwarded HTTP port  [default: True]",
-    default=None,
-)
-@option(
-    "--preemptible/--non-preemptible",
-    "-p/-P",
-    help="Run job on a lower-cost preemptible instance",
-    default=False,
-    show_default=True,
-)
-@option(
-    "-n",
-    "--name",
-    metavar="NAME",
-    type=JOB_NAME,
-    help="Optional job name",
-    default=None,
-    secure=True,
-)
-@option(
-    "--tag",
-    metavar="TAG",
-    type=str,
-    help="Optional job tag, multiple values allowed",
-    multiple=True,
-)
-@option(
-    "-d",
-    "--description",
-    metavar="DESC",
-    help="Optional job description in free format",
-    secure=True,
-)
-@deprecated_quiet_option
-@option(
-    "-v",
-    "--volume",
-    metavar="MOUNT",
-    multiple=True,
-    help=(
-        "Mounts directory from vault into container. "
-        "Use multiple options to mount more than one volume. "
-    ),
-    secure=True,
-)
-@option(
-    "--entrypoint",
-    type=str,
-    help=(
-        "Executable entrypoint in the container "
-        "(note that it overwrites `ENTRYPOINT` and `CMD` "
-        "instructions of the docker image)"
-    ),
-    secure=True,
-)
-@option(
-    "-e",
-    "--env",
-    metavar="VAR=VAL",
-    multiple=True,
-    help=(
-        "Set environment variable in container "
-        "Use multiple options to define more than one variable"
-    ),
-    secure=True,
-)
-@option(
-    "--env-file",
-    type=click.Path(exists=True),
-    multiple=True,
-    help="File with environment variables to pass",
-    secure=True,
-)
-@option(
-    "--restart",
-    default="never",
-    show_default=True,
-    type=click.Choice([str(i) for i in JobRestartPolicy]),
-    help="Restart policy to apply when a job exits",
-)
-@option(
-    "--life-span",
-    type=str,
-    metavar="TIMEDELTA",
-    help=(
-        "Optional job run-time limit in the format '1d2h3m4s' "
-        "(some parts may be missing). Set '0' to disable. "
-        "Default value '1d' can be changed in the user config."
-    ),
-)
-@option(
-    "--wait-start/--no-wait-start",
-    default=True,
-    show_default=True,
-    help="Wait for a job start or failure",
-)
-@option(
-    "--pass-config/--no-pass-config",
-    default=False,
-    show_default=True,
-    help="Upload neuro config to the job",
-)
-@option(
-    "--port-forward",
-    type=LOCAL_REMOTE_PORT,
-    multiple=True,
-    metavar="LOCAL_PORT:REMOTE_RORT",
-    help="Forward port(s) of a running job to local port(s) "
-    "(use multiple times for forwarding several ports)",
-)
-@option("--browse", is_flag=True, help="Open a job's URL in a web browser")
-@option(
-    "--detach",
-    is_flag=True,
-    help="Don't attach to job logs and don't wait for exit code",
-)
-@TTY_OPT
-async def submit(
-    root: Root,
-    image: RemoteImage,
-    gpu: Optional[int],
-    gpu_model: Optional[str],
-    tpu_type: Optional[str],
-    tpu_software_version: Optional[str],
-    cpu: float,
-    memory: int,
-    extshm: bool,
-    http: Optional[int],
-    http_auth: Optional[bool],
-    entrypoint: Optional[str],
-    cmd: Sequence[str],
-    working_dir: Optional[str],
-    volume: Sequence[str],
-    env: Sequence[str],
-    env_file: Sequence[str],
-    restart: str,
-    life_span: Optional[str],
-    port_forward: List[Tuple[int, int]],
-    preemptible: bool,
-    name: Optional[str],
-    tag: Sequence[str],
-    description: Optional[str],
-    wait_start: bool,
-    pass_config: bool,
-    browse: bool,
-    detach: bool,
-    tty: Optional[bool],
-) -> None:
-    """
-    Submit an image to run on the cluster.
-
-    IMAGE container image name.
-
-    CMD list will be passed as commands to model container.
-
-    Examples:
-
-    # Starts a container pytorch:latest with two paths mounted. Directory /q1/
-    # is mounted in read only mode to /qm directory within container.
-    # Directory /mod mounted to /mod directory in read-write mode.
-    neuro submit --volume storage:/q1:/qm:ro --volume storage:/mod:/mod:rw \
-      pytorch:latest
-
-    # Starts a container using the custom image my-ubuntu:latest stored in neuromation
-    # registry, run /script.sh and pass arg1 arg2 arg3 as its arguments:
-    neuro submit image:my-ubuntu:latest --entrypoint=/script.sh arg1 arg2 arg3
-    """
-    if tty is None:
-        tty = root.tty
-    await run_job(
-        root,
-        image=image,
-        gpu=gpu,
-        gpu_model=gpu_model,
-        tpu_type=tpu_type,
-        tpu_software_version=tpu_software_version,
-        cpu=cpu,
-        memory=memory,
-        extshm=extshm,
-        http=http,
-        http_auth=http_auth,
-        entrypoint=entrypoint,
-        cmd=cmd,
-        working_dir=working_dir,
-        volume=volume,
-        env=env,
-        env_file=env_file,
-        restart=restart,
-        life_span=life_span,
-        port_forward=port_forward,
-        preemptible=preemptible,
-        name=name,
-        tags=tag,
-        description=description,
-        wait_start=wait_start,
-        pass_config=pass_config,
-        browse=browse,
-        detach=detach,
-        tty=tty,
-        schedule_timeout=None,
-    )
-
-
 @command(context_settings=dict(allow_interspersed_args=False))
 @argument("job", type=JOB)
 @argument("cmd", nargs=-1, type=click.UNPROCESSED, required=True)
@@ -410,7 +137,9 @@ async def exec(
     """
     real_cmd = _parse_cmd(cmd)
     job = await resolve_job(
-        job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
+        job,
+        client=root.client,
+        status=JobStatus.active_items(),
     )
     if tty is None:
         tty = root.tty
@@ -463,7 +192,9 @@ async def port_forward(
             err=True,
         )
     job_id = await resolve_job(
-        job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
+        job,
+        client=root.client,
+        status=JobStatus.active_items(),
     )
     async with AsyncExitStack() as stack:
         for local_port, job_port in local_remote_port:
@@ -492,13 +223,7 @@ async def logs(root: Root, job: str) -> None:
     id = await resolve_job(
         job,
         client=root.client,
-        status={
-            JobStatus.PENDING,
-            JobStatus.RUNNING,
-            JobStatus.SUCCEEDED,
-            JobStatus.CANCELLED,
-            JobStatus.FAILED,
-        },
+        status=JobStatus.items(),
     )
     await process_logs(root, id, None)
 
@@ -520,17 +245,11 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
     id = await resolve_job(
         job,
         client=root.client,
-        status={
-            JobStatus.PENDING,
-            JobStatus.RUNNING,
-            JobStatus.SUCCEEDED,
-            JobStatus.CANCELLED,
-            JobStatus.FAILED,
-        },
+        status=JobStatus.items(),
     )
     status = await root.client.jobs.status(id)
-    progress = JobStartProgress.create(tty=root.tty, color=root.color, quiet=root.quiet)
-    while status.status == JobStatus.PENDING:
+    progress = JobStartProgress.create(console=root.console, quiet=root.quiet)
+    while status.status.is_pending:
         await asyncio.sleep(0.2)
         status = await root.client.jobs.status(id)
         progress.step(status)
@@ -545,7 +264,9 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
     "-s",
     "--status",
     multiple=True,
-    type=click.Choice(["pending", "running", "succeeded", "failed", "all"]),
+    type=click.Choice(
+        [item.value for item in JobStatus if item != JobStatus.UNKNOWN] + ["all"]
+    ),
     help=(
         "Filter out jobs by status (multiple option)."
         " Note: option `all` is deprecated, use `neuro ps -a` instead."
@@ -555,7 +276,8 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
     "-o",
     "--owner",
     multiple=True,
-    help="Filter out jobs by owner (multiple option).",
+    help="Filter out jobs by owner (multiple option). "
+    "Supports `ME` option to filter by the current user.",
     secure=True,
 )
 @option("-n", "--name", metavar="NAME", help="Filter out jobs by name.", secure=True)
@@ -590,10 +312,7 @@ async def attach(root: Root, job: str, port_forward: List[Tuple[int, int]]) -> N
     "--all",
     is_flag=True,
     default=False,
-    help=(
-        "Show all jobs regardless the status (equivalent to "
-        "`-s pending -s running -s succeeded -s failed`)."
-    ),
+    help="Show all jobs regardless the status.",
 )
 @deprecated_quiet_option
 @option("-w", "--wide", is_flag=True, help="Do not cut long lines for terminal width.")
@@ -640,6 +359,9 @@ async def ls(
 
     statuses = calc_statuses(status, all)
     owners = set(owner)
+    if "ME" in owners:
+        owners.remove("ME")
+        owners.add(root.client.config.username)
     tags = set(tag)
     jobs = root.client.jobs.list(
         statuses=statuses,
@@ -664,16 +386,13 @@ async def ls(
     if root.quiet:
         formatter: BaseJobsFormatter = SimpleJobsFormatter()
     else:
-        if wide or not root.tty:
-            width = 0
-        else:
-            width = root.terminal_size[0]
         image_fmtr = image_formatter(uri_formatter=uri_fmtr)
         formatter = TabularJobsFormatter(
-            width, root.client.username, format, image_formatter=image_fmtr
+            root.client.username, format, image_formatter=image_fmtr
         )
 
-    pager_maybe(formatter([job async for job in jobs]), root.tty, root.terminal_size)
+    with root.pager():
+        root.print(formatter([job async for job in jobs]))
 
 
 @command()
@@ -686,13 +405,7 @@ async def status(root: Root, job: str, full_uri: bool) -> None:
     id = await resolve_job(
         job,
         client=root.client,
-        status={
-            JobStatus.PENDING,
-            JobStatus.RUNNING,
-            JobStatus.SUCCEEDED,
-            JobStatus.CANCELLED,
-            JobStatus.FAILED,
-        },
+        status=JobStatus.items(),
     )
     res = await root.client.jobs.status(id)
     uri_fmtr: URIFormatter
@@ -702,11 +415,7 @@ async def status(root: Root, job: str, full_uri: bool) -> None:
         uri_fmtr = uri_formatter(
             username=root.client.username, cluster_name=root.client.cluster_name
         )
-    if root.tty:
-        width = root.terminal_size[0]
-    else:
-        width = 0
-    click.echo(JobStatusFormatter(uri_formatter=uri_fmtr, width=width)(res))
+    root.print(JobStatusFormatter(uri_formatter=uri_fmtr)(res))
 
 
 @command()
@@ -715,7 +424,12 @@ async def tags(root: Root) -> None:
     List all tags submitted by the user.
     """
     res = await root.client.jobs.tags()
-    pager_maybe(res, root.tty, root.terminal_size)
+    table = Table.grid()
+    table.add_column("")
+    for item in res:
+        table.add_row(item)
+    with root.pager():
+        root.print(table)
 
 
 @command()
@@ -724,9 +438,7 @@ async def browse(root: Root, job: str) -> None:
     """
     Opens a job's URL in a web browser.
     """
-    id = await resolve_job(
-        job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
-    )
+    id = await resolve_job(job, client=root.client, status=JobStatus.active_items())
     res = await root.client.jobs.status(id)
     await browse_job(root, res)
 
@@ -744,18 +456,11 @@ async def top(root: Root, job: str, timeout: float) -> None:
     """
     Display GPU/CPU/Memory usage.
     """
-    formatter = JobTelemetryFormatter()
-    id = await resolve_job(
-        job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
-    )
-    print_header = True
-    async with async_timeout.timeout(timeout if timeout else None):
-        async for res in root.client.jobs.top(id):
-            if print_header:
-                click.echo(formatter.header())
-                print_header = False
-            line = formatter(res)
-            click.echo(f"\r{line}", nl=False)
+    with JobTelemetryFormatter(root.console) as formatter:
+        id = await resolve_job(job, client=root.client, status=JobStatus.active_items())
+        async with async_timeout.timeout(timeout if timeout else None):
+            async for res in root.client.jobs.top(id):
+                formatter.update(res)
 
 
 @command()
@@ -773,13 +478,7 @@ async def save(root: Root, job: str, image: RemoteImage) -> None:
     id = await resolve_job(
         job,
         client=root.client,
-        status={
-            JobStatus.PENDING,
-            JobStatus.RUNNING,
-            JobStatus.SUCCEEDED,
-            JobStatus.CANCELLED,
-            JobStatus.FAILED,
-        },
+        status=JobStatus.items(),
     )
     progress = DockerImageProgress.create(console=root.console, quiet=root.quiet)
     with contextlib.closing(progress):
@@ -796,7 +495,7 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     errors = []
     for job in jobs:
         job_resolved = await resolve_job(
-            job, client=root.client, status={JobStatus.PENDING, JobStatus.RUNNING}
+            job, client=root.client, status=JobStatus.active_items()
         )
         try:
             await root.client.jobs.kill(job_resolved)
@@ -972,6 +671,12 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     show_default=True,
     help="Upload neuro config to the job",
 )
+@option(
+    "--wait-for-seat/--no-wait-for-seat",
+    default=False,
+    show_default=True,
+    help="Wait for total running jobs quota",
+)
 @option(
     "--port-forward",
     type=LOCAL_REMOTE_PORT,
@@ -986,6 +691,12 @@ async def kill(root: Root, jobs: Sequence[str]) -> None:
     is_flag=True,
     help="Don't attach to job logs and don't wait for exit code",
 )
+@option(
+    "--privileged",
+    default=False,
+    show_default=True,
+    help="Run job in privileged mode, if it is supported by cluster.",
+)
 @TTY_OPT
 async def run(
     root: Root,
@@ -1008,11 +719,13 @@ async def run(
     description: Optional[str],
     wait_start: bool,
     pass_config: bool,
+    wait_for_seat: bool,
     port_forward: List[Tuple[int, int]],
     browse: bool,
     detach: bool,
     tty: Optional[bool],
     schedule_timeout: Optional[str],
+    privileged: bool,
 ) -> None:
     """
     Run a job with predefined resources configuration.
@@ -1047,12 +760,7 @@ async def run(
     await run_job(
         root,
         image=image,
-        gpu=job_preset.gpu,
-        gpu_model=job_preset.gpu_model,
-        tpu_type=job_preset.tpu_type,
-        tpu_software_version=job_preset.tpu_software_version,
-        cpu=job_preset.cpu,
-        memory=job_preset.memory_mb,
+        preset=preset,
         extshm=extshm,
         http=http,
         http_auth=http_auth,
@@ -1065,21 +773,21 @@ async def run(
         restart=restart,
         life_span=life_span,
         port_forward=port_forward,
-        preemptible=job_preset.is_preemptible,
         name=name,
         tags=tag,
         description=description,
         wait_start=wait_start,
         pass_config=pass_config,
+        wait_for_jobs_quota=wait_for_seat,
         browse=browse,
         detach=detach,
         tty=tty,
         schedule_timeout=schedule_timeout,
+        privileged=privileged,
     )
 
 
 job.add_command(run)
-job.add_command(submit)
 job.add_command(ls)
 job.add_command(status)
 job.add_command(tags)
@@ -1101,12 +809,7 @@ async def run_job(
     root: Root,
     *,
     image: RemoteImage,
-    gpu: Optional[int],
-    gpu_model: Optional[str],
-    tpu_type: Optional[str],
-    tpu_software_version: Optional[str],
-    cpu: float,
-    memory: int,
+    preset: str,
     extshm: bool,
     http: Optional[int],
     http_auth: Optional[bool],
@@ -1119,16 +822,17 @@ async def run_job(
     restart: str,
     life_span: Optional[str],
     port_forward: List[Tuple[int, int]],
-    preemptible: bool,
     name: Optional[str],
     tags: Sequence[str],
     description: Optional[str],
     wait_start: bool,
     pass_config: bool,
+    wait_for_jobs_quota: bool,
     browse: bool,
     detach: bool,
     tty: bool,
     schedule_timeout: Optional[str],
+    privileged: bool,
 ) -> JobDescription:
     if http_auth is None:
         http_auth = True
@@ -1169,33 +873,26 @@ async def run_job(
 
     log.info(f"Using image '{image}'")
 
-    if tpu_type:
-        if not tpu_software_version:
-            raise ValueError(
-                "--tpu-sw-version cannot be empty while --tpu-type specified"
-            )
-    resources = Resources(
-        memory_mb=memory,
-        cpu=cpu,
-        gpu=gpu,
-        gpu_model=gpu_model,
-        shm=extshm,
-        tpu_type=tpu_type,
-        tpu_software_version=tpu_software_version,
-    )
-
     volume_parse_result = root.client.parse.volumes(volume)
     volumes = list(volume_parse_result.volumes)
     secret_files = volume_parse_result.secret_files
     disk_volumes = volume_parse_result.disk_volumes
 
     if pass_config:
-        env_name = NEURO_STEAL_CONFIG
+        env_name = PASS_CONFIG_ENV_NAME
         if env_name in env_dict:
             raise ValueError(f"{env_name} is already set to {env_dict[env_name]}")
+
+        # The following code is compatibility layer with old images
+        # TODO: remove this and upload_and_map_config function
+        old_env_name = "NEURO_STEAL_CONFIG"
+        if old_env_name in env_dict:
+            raise ValueError(f"{env_name} is already set to {env_dict[env_name]}")
+
         env_var, secret_volume = await upload_and_map_config(root)
-        env_dict[NEURO_STEAL_CONFIG] = env_var
+        env_dict[old_env_name] = env_var
         volumes.append(secret_volume)
+        # End of compatibility layer
 
     if volumes:
         log.info(
@@ -1203,38 +900,38 @@ async def run_job(
             + "\n".join(f"  {volume_to_verbose_str(v)}" for v in volumes)
         )
 
-    container = Container(
+    job = await root.client.jobs.start(
         image=image,
+        preset_name=preset,
         entrypoint=entrypoint,
         command=real_cmd,
         working_dir=working_dir,
         http=HTTPPort(http, http_auth) if http else None,
-        resources=resources,
         env=env_dict,
         volumes=volumes,
         secret_env=secret_env_dict,
         secret_files=secret_files,
         disk_volumes=disk_volumes,
         tty=tty,
-    )
-
-    job = await root.client.jobs.run(
-        container,
-        is_preemptible=preemptible,
+        shm=extshm,
+        pass_config=pass_config,
+        wait_for_jobs_quota=wait_for_jobs_quota,
         name=name,
         tags=tags,
         description=description,
         restart_policy=job_restart_policy,
         life_span=job_life_span,
         schedule_timeout=job_schedule_timeout,
+        privileged=privileged,
     )
-    progress = JobStartProgress.create(tty=root.tty, color=root.color, quiet=root.quiet)
-    progress.begin(job)
-    while wait_start and job.status == JobStatus.PENDING:
-        await asyncio.sleep(0.2)
-        job = await root.client.jobs.status(job.id)
-        progress.step(job)
-    progress.end(job)
+    with JobStartProgress.create(console=root.console, quiet=root.quiet) as progress:
+        progress.begin(job)
+        while wait_start and job.status == JobStatus.PENDING:
+            await asyncio.sleep(0.2)
+            job = await root.client.jobs.status(job.id)
+            progress.step(job)
+        progress.end(job)
+
     # Even if we detached, but the job has failed to start
     # (most common reason - no resources), the command fails
     if job.status == JobStatus.FAILED:
@@ -1299,7 +996,7 @@ async def browse_job(root: Root, job: JobDescription) -> None:
 
 
 def calc_statuses(status: Sequence[str], all: bool) -> Set[JobStatus]:
-    defaults = {"running", "pending"}
+    defaults = {item.value for item in JobStatus.active_items()}
     statuses = set(status)
 
     if "all" in statuses:
