@@ -2,6 +2,7 @@ import asyncio
 import io
 import logging
 import os
+import shutil
 import sys
 import warnings
 from pathlib import Path
@@ -57,7 +58,6 @@ from .utils import (
     option,
     pager_maybe,
     print_help,
-    steal_config_maybe,
 )
 
 
@@ -168,7 +168,17 @@ class MainGroup(Group):
                 option = "--hide-token" if kwargs["hide_token"] else "--no-hide-token"
                 raise click.UsageError(f"{option} requires --trace")
             hide_token_bool = kwargs["hide_token"]
-        steal_config_maybe(Path(kwargs["neuromation_config"]))
+
+        # The following code is compatibility layer with old images
+        # New client doesn't make use of NEURO_STEAL_CONFIG, but
+        # it is better to remove it from storage
+        # TODO: remove this and upload_and_map_config function
+        if "NEURO_STEAL_CONFIG" in os.environ:
+            path = Path(os.environ["NEURO_STEAL_CONFIG"])
+            if path.exists():
+                shutil.rmtree(path)
+        # End of compatibility layer
+
         root = Root(
             verbosity=verbosity,
             color=real_color,
@@ -489,7 +499,6 @@ cli.add_command(DeprecatedGroup(storage.storage, name="store", hidden=True))
 
 # shortcuts
 cli.add_command(job.run)
-cli.add_command(job.submit)
 cli.add_command(alias(job.ls, "ps", help=job.ls.help, deprecated=False))
 cli.add_command(job.status)
 cli.add_command(job.exec)
