@@ -58,7 +58,7 @@ class MaybePager(Pager):
             os.environ["LESS"] = "-R"
             click.echo_via_pager(content)
         else:
-            print(content)
+            print(content, end="")
 
 
 @dataclass
@@ -90,6 +90,33 @@ class Root:
             highlight=False,
             log_path=False,
         )
+        if not self.console.is_terminal or self.console.is_dumb_terminal:
+            # resize with wider width to prevent wrapping/cropping
+            self.console = Console(
+                color_system="auto" if self.color else None,
+                force_terminal=self.tty,
+                highlight=False,
+                log_path=False,
+                width=2048,
+            )
+
+        self.err_console = Console(
+            file=sys.stderr,
+            color_system="auto" if self.color else None,
+            force_terminal=self.tty,
+            highlight=False,
+            log_path=False,
+        )
+        if not self.err_console.is_terminal or self.err_console.is_dumb_terminal:
+            # resize with wider width to prevent wrapping/cropping
+            self.err_console = Console(
+                file=sys.stderr,
+                color_system="auto" if self.color else None,
+                force_terminal=self.tty,
+                highlight=False,
+                log_path=False,
+                width=2048,
+            )
 
     def close(self) -> None:
         if self._client is not None:
@@ -157,7 +184,7 @@ class Root:
         try:
             client = await self.init_client()
         except ConfigError:
-            return await load_user_config(self.config_path.expanduser())
+            return load_user_config(self.config_path.expanduser())
         else:
             return await client.config.get_user_config()
 
@@ -283,5 +310,8 @@ class Root:
     def pager(self) -> PagerContext:
         return self.console.pager(MaybePager(self.console), styles=True, links=True)
 
-    def print(self, *objects: Any, **kwargs: Any) -> None:
-        self.console.print(*objects, **kwargs)
+    def print(self, *objects: Any, err: bool = False, **kwargs: Any) -> None:
+        if err:
+            self.err_console.print(*objects, **kwargs)
+        else:
+            self.console.print(*objects, **kwargs)
