@@ -13,6 +13,7 @@ from neuromation.api.admin import (
     _NodePool,
     _Storage,
 )
+from neuromation.api.server_cfg import Preset
 from tests import _TestServerFactory
 
 
@@ -64,6 +65,8 @@ async def test_list_clusters_with_cloud_provider(
                             "idle_size": 1,
                             "available_cpu": 7,
                             "available_memory_mb": 46080,
+                            "disk_type": "ssd",
+                            "disk_size_gb": 150,
                             "gpu": 1,
                             "gpu_model": "nvidia-tesla-k80",
                             "is_tpu_enabled": True,
@@ -75,6 +78,7 @@ async def test_list_clusters_with_cloud_provider(
                             "max_size": 2,
                             "available_cpu": 7,
                             "available_memory_mb": 46080,
+                            "disk_size_gb": 150,
                         },
                     ],
                     "storage": {"description": "Filestore"},
@@ -92,6 +96,7 @@ async def test_list_clusters_with_cloud_provider(
                             "machine_type": "n1-highmem-8",
                             "available_cpu": 7,
                             "available_memory_mb": 46080,
+                            "disk_size_gb": 150,
                         },
                     ],
                     "storage": {"description": "NFS"},
@@ -137,6 +142,8 @@ async def test_list_clusters_with_cloud_provider(
                             machine_type="n1-highmem-8",
                             available_cpu=7.0,
                             available_memory_mb=46080,
+                            disk_type="ssd",
+                            disk_size_gb=150,
                             gpu=1,
                             gpu_model="nvidia-tesla-k80",
                             is_tpu_enabled=True,
@@ -148,6 +155,7 @@ async def test_list_clusters_with_cloud_provider(
                             machine_type="n1-highmem-8",
                             available_cpu=7.0,
                             available_memory_mb=46080,
+                            disk_size_gb=150,
                         ),
                     ],
                     storage=_Storage(description="Filestore"),
@@ -165,6 +173,7 @@ async def test_list_clusters_with_cloud_provider(
                             min_size=2,
                             max_size=2,
                             machine_type="n1-highmem-8",
+                            disk_size_gb=150,
                             available_cpu=7.0,
                             available_memory_mb=46080,
                         ),
@@ -413,9 +422,9 @@ async def test_set_user_quota(
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/api/v1")) as client:
-        await client._admin.set_user_quota("default", "ivan", 100, 200)
-        await client._admin.set_user_quota("neuro", "user2", None, None)
-        await client._admin.set_user_quota("neuro-ai", "user3", 150, None)
+        await client._admin.set_user_quota("default", "ivan", 10, 100, 200)
+        await client._admin.set_user_quota("neuro", "user2", None, None, None)
+        await client._admin.set_user_quota("neuro-ai", "user3", None, 150, None)
         assert requested_cluster_users == [
             ("default", "ivan"),
             ("neuro", "user2"),
@@ -424,6 +433,7 @@ async def test_set_user_quota(
         assert len(requested_payloads) == 3
         assert {
             "quota": {
+                "total_running_jobs": 10,
                 "total_gpu_run_time_minutes": 100,
                 "total_non_gpu_run_time_minutes": 200,
             },
@@ -503,3 +513,74 @@ async def test_get_cloud_provider_options(
     async with make_client(srv.make_url("/api/v1")) as client:
         result = await client._admin.get_cloud_provider_options("aws")
         assert result == sample_response
+
+
+async def test_update_cluster_resource_presets(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def update_cluster_resource_presets(
+        request: web.Request,
+    ) -> web.StreamResponse:
+        assert request.match_info["cluster_name"] == "my_cluster"
+        assert sorted(await request.json(), key=lambda x: x["name"]) == [
+            {
+                "name": "cpu-micro",
+                "cpu": 0.1,
+                "memory_mb": 100,
+                "is_preemptible": False,
+                "is_preemptible_node_required": False,
+            },
+            {
+                "name": "cpu-micro-p",
+                "cpu": 0.1,
+                "memory_mb": 100,
+                "is_preemptible": True,
+                "is_preemptible_node_required": True,
+            },
+            {
+                "name": "gpu-micro",
+                "cpu": 0.2,
+                "memory_mb": 200,
+                "gpu": 1,
+                "gpu_model": "nvidia-tesla-k80",
+                "is_preemptible": False,
+                "is_preemptible_node_required": False,
+            },
+            {
+                "name": "tpu-micro",
+                "cpu": 0.3,
+                "memory_mb": 300,
+                "tpu": {"type": "v2-8", "software_version": "1.14"},
+                "is_preemptible": False,
+                "is_preemptible_node_required": False,
+            },
+        ]
+        return web.Response(status=HTTPNoContent.status_code)
+
+    app = web.Application()
+    app.router.add_put(
+        "/api/v1/clusters/{cluster_name}/orchestrator/resource_presets",
+        update_cluster_resource_presets,
+    )
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/api/v1")) as client:
+        await client._admin.update_cluster_resource_presets(
+            "my_cluster",
+            {
+                "cpu-micro": Preset(cpu=0.1, memory_mb=100),
+                "cpu-micro-p": Preset(
+                    cpu=0.1,
+                    memory_mb=100,
+                    is_preemptible=True,
+                    is_preemptible_node_required=True,
+                ),
+                "gpu-micro": Preset(
+                    cpu=0.2, memory_mb=200, gpu=1, gpu_model="nvidia-tesla-k80"
+                ),
+                "tpu-micro": Preset(
+                    cpu=0.3, memory_mb=300, tpu_type="v2-8", tpu_software_version="1.14"
+                ),
+            },
+        )
