@@ -1,15 +1,18 @@
+import asyncio
 import base64
 import hashlib
 import os
+import time
 from datetime import datetime
 from pathlib import Path
-from typing import Any, AsyncIterator, Callable, Dict, List, NoReturn, Set
+from typing import Any, AsyncIterator, Callable, Dict, List, NoReturn, Optional, Set
 from unittest import mock
 
 import pytest
 from aiohttp import web
 from yarl import URL
 
+import neuromation.api.blob_storage
 from neuromation.api import (
     Action,
     BlobListing,
@@ -81,44 +84,44 @@ def dir_list(path: Path) -> List[Dict[str, Any]]:
 
 @pytest.fixture
 def blob_storage_contents() -> _ContentsObj:
-    mtime1 = datetime(2019, 1, 1)
-    mtime2 = datetime(2019, 1, 2)
+    mtime1 = datetime(2019, 1, 1).timestamp()
+    mtime2 = datetime(2019, 1, 2).timestamp()
 
     contents: Dict[str, Dict[str, Any]] = {
         "empty/": {
             "key": "empty/",
             "size": 0,
-            "last_modified": mtime1.timestamp(),
+            "last_modified": mtime1,
             "body": b"",
         },
         "folder1/xxx.txt": {
             "key": "folder1/xxx.txt",
             "size": 1,
-            "last_modified": mtime1.timestamp(),
+            "last_modified": mtime1,
             "body": b"w",
         },
         "folder1/yyy.json": {
             "key": "folder1/yyy.json",
             "size": 2,
-            "last_modified": mtime2.timestamp(),
+            "last_modified": mtime2,
             "body": b"bb",
         },
         "test.json": {
             "key": "test.json",
             "size": 213,
-            "last_modified": mtime1.timestamp(),
+            "last_modified": mtime1,
             "body": b"w" * 213,
         },
         "test1.txt": {
             "key": "test1.txt",
             "size": 111,
-            "last_modified": mtime1.timestamp(),
+            "last_modified": mtime1,
             "body": b"w" * 111,
         },
         "test2.txt": {
             "key": "test2.txt",
             "size": 222,
-            "last_modified": mtime2.timestamp(),
+            "last_modified": mtime2,
             "body": b"w" * 222,
         },
     }
@@ -190,15 +193,29 @@ async def blob_storage_server(
             raise web.HTTPNotFound()
         blob = CONTENTS[key]
 
+        rng = request.http_range
+        content = blob["body"]
         resp = web.StreamResponse(status=200)
         etag = hashlib.md5(blob["body"]).hexdigest()
         resp.headers.update({"ETag": repr(etag)})
-        resp.last_modified = blob["last_modified"]
-        resp.content_length = len(blob["body"])
+        resp.last_modified = int(blob["last_modified"])  # type: ignore
+        start, stop, _ = rng.indices(len(content))
+        if not (rng.start is rng.stop is None):
+            if start >= stop:
+                raise RuntimeError
+            resp.set_status(web.HTTPPartialContent.status_code)
+            resp.headers["Content-Range"] = f"bytes {start}-{stop-1}/{len(content)}"
+        resp.content_length = stop - start
         resp.content_type = "application/octet-stream"
         await resp.prepare(request)
         if request.method != "HEAD":
-            await resp.write(blob["body"])
+            chunk_size = max(len(content) * 2 // 5, 200)
+            if stop - start > chunk_size:
+                await resp.write(content[start : start + chunk_size])
+                raise RuntimeError
+            else:
+                await resp.write(content[start:stop])
+                await resp.write_eof()
         return resp
 
     async def put_blob(request: web.Request) -> web.Response:
@@ -210,7 +227,7 @@ async def blob_storage_server(
         blob = {
             "key": key,
             "size": len(body),
-            "last_modified": datetime.now().timestamp(),
+            "last_modified": time.time(),
             "body": body,
         }
         CONTENTS[key] = blob
@@ -485,7 +502,7 @@ async def test_blob_storage_head_blob(
 ) -> None:
     bucket_name = "foo"
     key = "text.txt"
-    mtime1 = datetime.now()
+    mtime1 = time.time()
 
     async def handler(request: web.Request) -> web.StreamResponse:
         assert "b3" in request.headers
@@ -493,7 +510,7 @@ async def test_blob_storage_head_blob(
         assert request.match_info == {"bucket": bucket_name, "path": key}
         resp = web.StreamResponse(status=200)
         resp.headers.update({"ETag": '"12312908asd"'})
-        resp.last_modified = mtime1
+        resp.last_modified = int(mtime1)  # type: ignore
         resp.content_length = 111
         resp.content_type = "plain/text"
         return resp
@@ -509,7 +526,7 @@ async def test_blob_storage_head_blob(
     assert ret == BlobListing(
         key=key,
         size=111,
-        modification_time=int(mtime1.timestamp()),
+        modification_time=int(mtime1),
         bucket_name=bucket_name,
     )
 
@@ -519,7 +536,7 @@ async def test_blob_storage_get_blob(
 ) -> None:
     bucket_name = "foo"
     key = "text.txt"
-    mtime1 = datetime.now()
+    mtime1 = time.time()
     body = b"W" * 1000
 
     async def handler(request: web.Request) -> web.StreamResponse:
@@ -528,7 +545,7 @@ async def test_blob_storage_get_blob(
         assert request.match_info == {"bucket": bucket_name, "path": key}
         resp = web.StreamResponse(status=200)
         resp.headers.update({"ETag": '"12312908asd"'})
-        resp.last_modified = mtime1
+        resp.last_modified = int(mtime1)  # type: ignore
         resp.content_length = len(body)
         resp.content_type = "plain/text"
         await resp.prepare(request)
@@ -545,7 +562,7 @@ async def test_blob_storage_get_blob(
             assert ret.stats == BlobListing(
                 key=key,
                 size=1000,
-                modification_time=int(mtime1.timestamp()),
+                modification_time=int(mtime1),
                 bucket_name=bucket_name,
             )
             assert await ret.body_stream.read() == body
@@ -556,7 +573,7 @@ async def test_blob_storage_fetch_blob(
 ) -> None:
     bucket_name = "foo"
     key = "text.txt"
-    mtime1 = datetime.now()
+    mtime1 = time.time()
     body = b"W" * 10 * 1024 * 1024
 
     async def handler(request: web.Request) -> web.StreamResponse:
@@ -565,7 +582,7 @@ async def test_blob_storage_fetch_blob(
         assert request.match_info == {"bucket": bucket_name, "path": key}
         resp = web.StreamResponse(status=200)
         resp.headers.update({"ETag": '"12312908asd"'})
-        resp.last_modified = mtime1
+        resp.last_modified = int(mtime1)  # type: ignore
         resp.content_length = len(body)
         resp.content_type = "plain/text"
         await resp.prepare(request)
@@ -584,6 +601,93 @@ async def test_blob_storage_fetch_blob(
         assert buf == body
 
 
+async def test_blob_storage_fetch_blob_partial_read(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    bucket_name = "foo"
+    key = "text.txt"
+    mtime1 = time.time()
+    body = b"ababahalamaha"
+
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert "b3" in request.headers
+        assert request.path == f"/blob/o/{bucket_name}/{key}"
+        assert request.match_info == {"bucket": bucket_name, "path": key}
+        rng = request.http_range
+        start, stop, _ = rng.indices(len(body))
+        resp = web.StreamResponse(status=web.HTTPPartialContent.status_code)
+        resp.headers["ETag"] = '"12312908asd"'
+        resp.headers["Content-Range"] = f"bytes {start}-{stop-1}/{len(body)}"
+        resp.last_modified = int(mtime1)  # type: ignore
+        resp.content_length = stop - start
+        resp.content_type = "plain/text"
+        await resp.prepare(request)
+        await resp.write(body[start:stop])
+        return resp
+
+    app = web.Application()
+    app.router.add_get(BlobUrlRotes.GET_OBJECT, handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        buf = bytearray()
+        async for chunk in client.blob_storage.fetch_blob(bucket_name, key, 5):
+            buf.extend(chunk)
+        assert buf == b"halamaha"
+
+        buf = bytearray()
+        async for chunk in client.blob_storage.fetch_blob(bucket_name, key, 5, 4):
+            buf.extend(chunk)
+        assert buf == b"hala"
+
+        buf = bytearray()
+        async for chunk in client.blob_storage.fetch_blob(bucket_name, key, 5, 20):
+            buf.extend(chunk)
+        assert buf == b"halamaha"
+
+        with pytest.raises(ValueError):
+            async for chunk in client.blob_storage.fetch_blob(bucket_name, key, 5, 0):
+                pass
+
+
+async def test_blob_storage_fetch_blob_unsupported_partial_read(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    bucket_name = "foo"
+    key = "text.txt"
+    mtime1 = time.time()
+    body = b"ababahalamaha"
+
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert "b3" in request.headers
+        assert request.path == f"/blob/o/{bucket_name}/{key}"
+        assert request.match_info == {"bucket": bucket_name, "path": key}
+        resp = web.StreamResponse(status=200)
+        resp.headers.update({"ETag": '"12312908asd"'})
+        resp.last_modified = int(mtime1)  # type: ignore
+        resp.content_length = len(body)
+        resp.content_type = "plain/text"
+        await resp.prepare(request)
+        await resp.write(body)
+        return resp
+
+    app = web.Application()
+    app.router.add_get(BlobUrlRotes.GET_OBJECT, handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        buf = b""
+        async for data in client.blob_storage.fetch_blob(bucket_name, key):
+            buf += data
+        assert buf == body
+
+        with pytest.raises(RuntimeError):
+            async for data in client.blob_storage.fetch_blob(bucket_name, key, 5):
+                pass
+
+
 async def test_blob_storage_put_blob(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -1237,6 +1341,240 @@ async def test_blob_storage_glob_blobs(
         assert ret == expected_keys
 
 
+@pytest.fixture
+def zero_time_threshold(monkeypatch: Any) -> None:
+    monkeypatch.setattr(neuromation.api.blob_storage, "TIME_THRESHOLD", 0.0)
+
+
+@pytest.fixture
+def small_block_size(monkeypatch: Any) -> None:
+    monkeypatch.setattr(neuromation.api.blob_storage, "READ_SIZE", 300)
+
+
+async def test_storage_upload_file_update(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+    small_block_size: None,
+) -> None:
+    # storage_file = storage_path / "file.txt"
+    local_file = tmp_path / "file.txt"
+    src = URL(local_file.as_uri())
+    dst = URL("blob:foo/file.txt")
+
+    # No destination file
+    assert blob_storage_contents.get("file.txt") is None
+    local_file.write_bytes(b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_file(src, dst, update=True)
+    assert blob_storage_contents["file.txt"]["body"] == b"old content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_file(src, dst, update=True)
+    assert blob_storage_contents["file.txt"]["body"] == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    add_blob(blob_storage_contents, "file.txt", b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_file(src, dst, update=True)
+    assert blob_storage_contents["file.txt"]["body"] == b"old"
+
+
+async def test_storage_upload_dir_update(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+) -> None:
+    local_dir = tmp_path / "folder"
+    local_file = local_dir / "nested" / "file.txt"
+    local_file.parent.mkdir(parents=True)
+    src = URL(local_dir.as_uri())
+    dst = URL("blob:foo")
+
+    # No destination file
+    assert blob_storage_contents.get("nested/file.txt") is None
+    # assert not storage_file.exists()
+    local_file.write_bytes(b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_dir(src, dst, update=True)
+    assert blob_storage_contents["nested/file.txt"]["body"] == b"old content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_dir(src, dst, update=True)
+    assert blob_storage_contents["nested/file.txt"]["body"] == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    add_blob(blob_storage_contents, "nested/file.txt", b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.upload_dir(src, dst, update=True)
+    assert blob_storage_contents["nested/file.txt"]["body"] == b"old"
+
+
+async def test_storage_download_file_update(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+) -> None:
+    local_file = tmp_path / "file.txt"
+    src = URL("blob:foo/file.txt")
+    dst = URL(local_file.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    add_blob(blob_storage_contents, "file.txt", b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, update=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    add_blob(blob_storage_contents, "file.txt", b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, update=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, update=True)
+    assert local_file.read_bytes() == b"old"
+
+
+async def test_storage_download_file_continue(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+    small_block_size: None,
+) -> None:
+    local_file = tmp_path / "file.txt"
+    src = URL("blob:foo/file.txt")
+    dst = URL(local_file.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    add_blob(blob_storage_contents, "file.txt", b"content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"content"
+
+    # Source file is newer
+    add_blob(blob_storage_contents, "file.txt", b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    local_file.write_bytes(b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    local_file.write_bytes(b"old long content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+
+async def test_storage_download_dir_update(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+) -> None:
+    local_dir = tmp_path / "folder"
+    local_file = local_dir / "nested" / "file.txt"
+    src = URL("blob:foo")
+    dst = URL(local_dir.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    add_blob(blob_storage_contents, "nested/file.txt", b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, update=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    add_blob(blob_storage_contents, "nested/file.txt", b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, update=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, update=True)
+    assert local_file.read_bytes() == b"old"
+
+
+async def test_storage_download_dir_continue(
+    blob_storage_server: Any,
+    make_client: _MakeClient,
+    blob_storage_contents: _ContentsObj,
+    tmp_path: Path,
+    zero_time_threshold: None,
+) -> None:
+    local_dir = tmp_path / "folder"
+    local_file = local_dir / "nested" / "file.txt"
+    src = URL("blob:foo")
+    dst = URL(local_dir.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    add_blob(blob_storage_contents, "nested/file.txt", b"content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"content"
+
+    # Source file is newer
+    add_blob(blob_storage_contents, "nested/file.txt", b"new content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    local_file.write_bytes(b"old")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    local_file.write_bytes(b"old long content")
+    async with make_client(blob_storage_server.make_url("/")) as client:
+        await client.blob_storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+
 async def test_storage_upload_dir_with_ignore_file_names(
     blob_storage_server: Any,
     make_client: _MakeClient,
@@ -1269,3 +1607,15 @@ async def test_storage_upload_dir_with_ignore_file_names(
         "folder/three",
         "folder/two",
     ]
+
+
+def add_blob(
+    contents: Dict[str, Any], key: str, body: bytes, mtime: Optional[float] = None
+) -> None:
+    contents[key] = {}
+    contents[key]["key"] = key
+    contents[key]["body"] = body
+    contents[key]["size"] = len(body)
+    if mtime is None:
+        mtime = time.time()
+    contents[key]["last_modified"] = mtime
