@@ -27,7 +27,6 @@ from neuromation.api.jobs import (
     _job_description_from_api,
 )
 from neuromation.api.parser import SecretFile
-from neuromation.api.server_cfg import Preset
 from tests import _TestServerFactory
 
 
@@ -471,6 +470,8 @@ async def test_status_failed(
             "description": "Not enough coffee",
         },
         "is_preemptible": True,
+        "is_preemptible_node_required": True,
+        "pass_config": False,
         "owner": "owner",
         "cluster_name": "default",
         "uri": "job://default/owner/job-id",
@@ -512,6 +513,7 @@ async def test_status_failed(
         ret = await client.jobs.status("job-id")
 
         assert ret == _job_description_from_api(JSON, client.parse)
+        assert ret.is_preemptible_node_required
 
 
 async def test_status_with_http(
@@ -530,6 +532,7 @@ async def test_status_with_http(
             "description": "Everything is fine",
         },
         "is_preemptible": True,
+        "pass_config": False,
         "owner": "owner",
         "cluster_name": "default",
         "uri": "job://default/owner/job-id",
@@ -589,6 +592,7 @@ async def test_status_with_tpu(
             "description": "Everything is fine",
         },
         "is_preemptible": True,
+        "pass_config": False,
         "owner": "owner",
         "cluster_name": "default",
         "uri": "job://default/owner/job-id",
@@ -635,6 +639,158 @@ async def test_status_with_tpu(
         assert ret.container.resources.tpu_software_version == "1.14"
 
 
+async def test_job_start(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    JSON = {
+        "id": "job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "status": "failed",
+        "history": {
+            "status": "failed",
+            "reason": "Error",
+            "description": "Mounted on Avail\\n/dev/shm     " "64M\\n\\nExit code: 1",
+            "created_at": "2018-09-25T12:28:21.298672+00:00",
+            "started_at": "2018-09-25T12:28:59.759433+00:00",
+            "finished_at": "2018-09-25T12:28:59.759433+00:00",
+        },
+        "owner": "owner",
+        "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "container": {
+            "image": "gcr.io/light-reality-205619/ubuntu:latest",
+            "command": "date",
+            "resources": {
+                "cpu": 1.0,
+                "memory_mb": 16384,
+                "gpu": 1,
+                "shm": False,
+                "gpu_model": "nvidia-tesla-p4",
+            },
+        },
+        "http_url": "http://my_host:8889",
+        "is_preemptible": False,
+        "pass_config": False,
+    }
+
+    async def handler(request: web.Request) -> web.Response:
+        data = await request.json()
+        assert data == {
+            "image": "submit-image-name",
+            "command": "submit-command",
+            "http": {"port": 8181, "requires_auth": True},
+            "resources": {
+                "shm": True,
+            },
+            "volumes": [
+                {
+                    "src_storage_uri": "storage://test-user/path_read_only",
+                    "dst_path": "/container/read_only",
+                    "read_only": True,
+                },
+                {
+                    "src_storage_uri": "storage://test-user/path_read_write",
+                    "dst_path": "/container/path_read_write",
+                    "read_only": False,
+                },
+            ],
+            "pass_config": False,
+            "cluster_name": "default",
+            "preset_name": "cpu-small",
+        }
+
+        return web.json_response(JSON)
+
+    app = web.Application()
+    app.router.add_post("/jobs", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        volumes: List[Volume] = [
+            Volume(
+                URL("storage://test-user/path_read_only"), "/container/read_only", True
+            ),
+            Volume(
+                URL("storage://test-user/path_read_write"),
+                "/container/path_read_write",
+                False,
+            ),
+        ]
+        ret = await client.jobs.start(
+            image=RemoteImage.new_external_image(name="submit-image-name"),
+            command="submit-command",
+            volumes=volumes,
+            http=HTTPPort(8181),
+            shm=True,
+            preset_name="cpu-small",
+        )
+
+        assert ret == _job_description_from_api(JSON, client.parse)
+
+
+async def test_job_start_with_privileged_flag(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    JSON = {
+        "id": "job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "status": "failed",
+        "history": {
+            "status": "failed",
+            "reason": "Error",
+            "description": "Mounted on Avail\\n/dev/shm     " "64M\\n\\nExit code: 1",
+            "created_at": "2018-09-25T12:28:21.298672+00:00",
+            "started_at": "2018-09-25T12:28:59.759433+00:00",
+            "finished_at": "2018-09-25T12:28:59.759433+00:00",
+        },
+        "owner": "owner",
+        "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "container": {
+            "image": "gcr.io/light-reality-205619/ubuntu:latest",
+            "command": "date",
+            "resources": {
+                "cpu": 1.0,
+                "memory_mb": 16384,
+                "gpu": 1,
+                "shm": False,
+                "gpu_model": "nvidia-tesla-p4",
+            },
+        },
+        "http_url": "http://my_host:8889",
+        "is_preemptible": False,
+        "pass_config": False,
+        "privileged": True,
+    }
+
+    async def handler(request: web.Request) -> web.Response:
+        data = await request.json()
+        assert data == {
+            "image": "submit-image-name",
+            "command": "submit-command",
+            "pass_config": False,
+            "cluster_name": "default",
+            "preset_name": "cpu-small",
+            "privileged": True,
+        }
+
+        return web.json_response(JSON)
+
+    app = web.Application()
+    app.router.add_post("/jobs", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        ret = await client.jobs.start(
+            image=RemoteImage.new_external_image(name="submit-image-name"),
+            command="submit-command",
+            preset_name="cpu-small",
+            privileged=True,
+        )
+
+        assert ret == _job_description_from_api(JSON, client.parse)
+
+
 async def test_job_run(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -665,6 +821,7 @@ async def test_job_run(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -695,6 +852,7 @@ async def test_job_run(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
 
@@ -729,6 +887,80 @@ async def test_job_run(
         assert ret == _job_description_from_api(JSON, client.parse)
 
 
+async def test_job_run_with_wait_for_quota(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    JSON = {
+        "id": "job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "status": "failed",
+        "history": {
+            "status": "failed",
+            "reason": "Error",
+            "description": "Mounted on Avail\\n/dev/shm     " "64M\\n\\nExit code: 1",
+            "created_at": "2018-09-25T12:28:21.298672+00:00",
+            "started_at": "2018-09-25T12:28:59.759433+00:00",
+            "finished_at": "2018-09-25T12:28:59.759433+00:00",
+        },
+        "owner": "owner",
+        "cluster_name": "default",
+        "uri": "job://default/owner/job-cf519ed3-9ea5-48f6-a8c5-492b810eb56f",
+        "container": {
+            "image": "gcr.io/light-reality-205619/ubuntu:latest",
+            "command": "date",
+            "resources": {
+                "cpu": 1.0,
+                "memory_mb": 16384,
+                "gpu": 1,
+                "shm": False,
+                "gpu_model": "nvidia-tesla-p4",
+            },
+        },
+        "http_url": "http://my_host:8889",
+        "is_preemptible": False,
+        "pass_config": False,
+    }
+
+    async def handler(request: web.Request) -> web.Response:
+        data = await request.json()
+        assert data == {
+            "container": {
+                "image": "submit-image-name",
+                "command": "submit-command",
+                "resources": {
+                    "memory_mb": 16384,
+                    "cpu": 7.0,
+                    "shm": True,
+                    "gpu": 1,
+                    "gpu_model": "test-gpu-model",
+                },
+            },
+            "is_preemptible": False,
+            "wait_for_jobs_quota": True,
+            "pass_config": False,
+            "cluster_name": "default",
+        }
+
+        return web.json_response(JSON)
+
+    app = web.Application()
+    app.router.add_post("/jobs", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        resources = Resources(16384, 7, 1, "test-gpu-model", True, None, None)
+        container = Container(
+            image=RemoteImage.new_external_image(name="submit-image-name"),
+            command="submit-command",
+            resources=resources,
+        )
+        ret = await client.jobs.run(
+            container=container, is_preemptible=False, wait_for_jobs_quota=True
+        )
+
+        assert ret == _job_description_from_api(JSON, client.parse)
+
+
 async def test_job_run_with_name_and_description(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -761,6 +993,7 @@ async def test_job_run_with_name_and_description(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -791,6 +1024,7 @@ async def test_job_run_with_name_and_description(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "name": "test-job-name",
             "description": "job description",
             "cluster_name": "default",
@@ -863,6 +1097,7 @@ async def test_job_run_with_tags(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -893,6 +1128,7 @@ async def test_job_run_with_tags(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "tags": ["t1", "t2", "t3"],
             "cluster_name": "default",
         }
@@ -963,6 +1199,7 @@ async def test_job_run_no_volumes(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -981,6 +1218,7 @@ async def test_job_run_no_volumes(
                 },
             },
             "is_preemptible": False,
+            "pass_config": False,
             "name": "test-job-name",
             "description": "job description",
             "cluster_name": "default",
@@ -1041,6 +1279,7 @@ async def test_job_run_with_relative_volume_uris(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -1076,6 +1315,7 @@ async def test_job_run_with_relative_volume_uris(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
 
@@ -1143,6 +1383,7 @@ async def test_job_run_with_secret_uris(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -1176,6 +1417,7 @@ async def test_job_run_with_secret_uris(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
 
@@ -1246,6 +1488,7 @@ async def test_job_run_with_disk_volume_uris(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -1271,6 +1514,7 @@ async def test_job_run_with_disk_volume_uris(
                 ],
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
 
@@ -1328,6 +1572,7 @@ async def test_job_run_preemptible(
             },
         },
         "is_preemptible": True,
+        "pass_config": False,
         "http_url": "http://my_host:8889",
     }
 
@@ -1359,6 +1604,7 @@ async def test_job_run_preemptible(
                 ],
             },
             "is_preemptible": True,
+            "pass_config": False,
             "name": "test-job-name",
             "description": "job description",
             "cluster_name": "default",
@@ -1430,6 +1676,7 @@ async def test_job_run_schedule_timeout(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
         "schedule_timeout": 5,
     }
 
@@ -1448,6 +1695,7 @@ async def test_job_run_schedule_timeout(
                 },
             },
             "is_preemptible": False,
+            "pass_config": False,
             "schedule_timeout": 5,
             "cluster_name": "default",
         }
@@ -1502,6 +1750,7 @@ async def test_job_run_tpu(
         },
         "http_url": "http://my_host:8889",
         "is_preemptible": False,
+        "pass_config": False,
     }
 
     async def handler(request: web.Request) -> web.Response:
@@ -1520,6 +1769,7 @@ async def test_job_run_tpu(
                 },
             },
             "is_preemptible": False,
+            "pass_config": False,
             "schedule_timeout": 5,
             "cluster_name": "default",
         }
@@ -1561,6 +1811,7 @@ async def test_job_run_with_tty(
                 "tty": True,
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
 
@@ -1615,6 +1866,7 @@ def create_job_response(
             },
         },
         "is_preemptible": True,
+        "pass_config": False,
         "owner": owner,
         "cluster_name": "default",
         "uri": f"job://default/{owner}/{id}",
@@ -2066,6 +2318,7 @@ async def test_job_run_life_span(
                 "command": "submit-command",
             },
             "is_preemptible": False,
+            "pass_config": False,
             "max_run_time_minutes": 10,
             "cluster_name": "default",
         }
@@ -2098,6 +2351,7 @@ async def test_job_run_restart_policy(
                 "command": "submit-command",
             },
             "is_preemptible": False,
+            "pass_config": False,
             "restart_policy": "always",
             "cluster_name": "default",
         }
@@ -2133,6 +2387,7 @@ async def test_job_run_working_dir(
                 "working_dir": "/working/dir",
             },
             "is_preemptible": False,
+            "pass_config": False,
             "cluster_name": "default",
         }
         result = create_job_response("job-id-1", "running")
@@ -2229,33 +2484,13 @@ async def test_get_capacity(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
     async def handler(request: web.Request) -> web.Response:
-        payload = await request.json()
-        assert payload == {
-            "gpu-p": {
-                "cpu": 0.1,
-                "memory_mb": 100,
-                "gpu": 1,
-                "gpu_model": "nvidia-tesla-k80",
-                "is_preemptible": True,
-            }
-        }
         return web.json_response({"cpu-micro": 10})
 
     app = web.Application()
-    app.router.add_post("/jobs/available", handler)
+    app.router.add_get("/jobs/capacity", handler)
 
     srv = await aiohttp_server(app)
 
     async with make_client(srv.make_url("/")) as client:
-        result = await client.jobs.get_capacity(
-            {
-                "gpu-p": Preset(
-                    cpu=0.1,
-                    memory_mb=100,
-                    gpu=1,
-                    gpu_model="nvidia-tesla-k80",
-                    is_preemptible=True,
-                )
-            }
-        )
+        result = await client.jobs.get_capacity()
         assert result == {"cpu-micro": 10}
