@@ -24,6 +24,7 @@ from neuromation.api import (
     StorageProgressStep,
 )
 from neuromation.api.abc import StorageProgressDelete
+from neuromation.api.storage import _parse_content_range
 from tests import _RawTestServerFactory, _TestServerFactory
 
 
@@ -47,6 +48,11 @@ def calc_diff(dcmp: "dircmp[str]", *, pre: str = "") -> List[Tuple[str, str]]:
     return ret
 
 
+@pytest.fixture
+def small_block_size(monkeypatch: Any) -> None:
+    monkeypatch.setattr(neuromation.api.storage, "READ_SIZE", 300)
+
+
 @pytest.fixture
 def storage_path(tmp_path: Path) -> Path:
     ret = tmp_path / "storage"
@@ -74,8 +80,39 @@ async def storage_server(
             content = await request.read()
             local_path.write_bytes(content)
             return web.Response(status=201)
+
+        elif op == "WRITE":
+            rng = _parse_content_range(request.headers.get("Content-Range"))
+            content = await request.read()
+            assert rng.stop - rng.start == len(content)
+            with open(local_path, "r+b") as f:
+                f.seek(rng.start)
+                f.write(content)
+            return web.Response(status=200)
+
         elif op == "OPEN":
-            return web.Response(body=local_path.read_bytes())
+            rng = request.http_range
+            content = local_path.read_bytes()
+            response = web.StreamResponse()
+            start, stop, _ = rng.indices(len(content))
+            if not (rng.start is rng.stop is None):
+                if start >= stop:
+                    raise RuntimeError
+                response.set_status(web.HTTPPartialContent.status_code)
+                response.headers[
+                    "Content-Range"
+                ] = f"bytes {start}-{stop-1}/{len(content)}"
+                response.content_length = stop - start
+            await response.prepare(request)
+            chunk_size = 200
+            if stop - start > chunk_size:
+                await response.write(content[start : start + chunk_size])
+                raise RuntimeError
+            else:
+                await response.write(content[start:stop])
+                await response.write_eof()
+                return response
+
         elif op == "GETFILESTATUS":
             if not local_path.exists():
                 raise web.HTTPNotFound()
@@ -91,6 +128,7 @@ async def storage_server(
                     }
                 }
             )
+
         elif op == "MKDIRS":
             try:
                 local_path.mkdir(parents=True, exist_ok=True)
@@ -100,6 +138,7 @@ async def storage_server(
                     content_type="application/json",
                 )
             return web.Response(status=201)
+
         elif op == "LISTSTATUS":
             if not local_path.exists():
                 raise web.HTTPNotFound()
@@ -116,6 +155,7 @@ async def storage_server(
                     }
                 )
             return await make_listiter_response(request, ret)
+
         else:
             raise web.HTTPInternalServerError(text=f"Unsupported operation {op}")
 
@@ -741,6 +781,27 @@ async def test_storage_create(
         await client.storage.create(URL("storage:file"), gen())
 
 
+async def test_storage_write(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.Response:
+        assert request.path == "/storage/user/file"
+        assert request.query == {"op": "WRITE"}
+        rng = _parse_content_range(request.headers.get("Content-Range"))
+        assert rng == slice(4, 9)
+        content = await request.read()
+        assert content == b"01234"
+        return web.Response(status=200)
+
+    app = web.Application()
+    app.router.add_patch("/storage/user/file", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        await client.storage.write(URL("storage:file"), b"01234", 4)
+
+
 async def test_storage_stats(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -787,18 +848,6 @@ async def test_storage_open(
             for i in range(5):
                 await resp.write(str(i).encode("ascii"))
             return resp
-        elif request.query["op"] == "GETFILESTATUS":
-            return web.json_response(
-                {
-                    "FileStatus": {
-                        "path": "file",
-                        "type": "FILE",
-                        "length": 5,
-                        "modificationTime": 3456,
-                        "permission": "read",
-                    }
-                }
-            )
         else:
             raise AssertionError(f"Unknown operation {request.query['op']}")
 
@@ -814,6 +863,80 @@ async def test_storage_open(
         assert buf == b"01234"
 
 
+async def test_storage_open_partial_read(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert request.path == "/storage/user/file"
+        if request.query["op"] == "OPEN":
+            rng = request.http_range
+            data = b"ababahalamaha"
+            start, stop, _ = rng.indices(len(data))
+            return web.Response(
+                status=web.HTTPPartialContent.status_code,
+                headers={"Content-Range": f"bytes {start}-{stop-1}/{len(data)}"},
+                body=data[start:stop],
+            )
+        else:
+            raise AssertionError(f"Unknown operation {request.query['op']}")
+
+    app = web.Application()
+    app.router.add_get("/storage/user/file", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        buf = bytearray()
+        async for chunk in client.storage.open(URL("storage:file"), 5):
+            buf.extend(chunk)
+        assert buf == b"halamaha"
+
+        buf = bytearray()
+        async for chunk in client.storage.open(URL("storage:file"), 5, 4):
+            buf.extend(chunk)
+        assert buf == b"hala"
+
+        buf = bytearray()
+        async for chunk in client.storage.open(URL("storage:file"), 5, 20):
+            buf.extend(chunk)
+        assert buf == b"halamaha"
+
+        buf = bytearray()
+        async for chunk in client.storage.open(URL("storage:file"), 5, 0):
+            buf.extend(chunk)
+        assert buf == b""
+
+
+async def test_storage_open_unsupported_partial_read(
+    aiohttp_server: _TestServerFactory, make_client: _MakeClient
+) -> None:
+    async def handler(request: web.Request) -> web.StreamResponse:
+        assert request.path == "/storage/user/file"
+        if request.query["op"] == "OPEN":
+            resp = web.StreamResponse()
+            await resp.prepare(request)
+            for i in range(5):
+                await resp.write(str(i).encode("ascii"))
+            return resp
+        else:
+            raise AssertionError(f"Unknown operation {request.query['op']}")
+
+    app = web.Application()
+    app.router.add_get("/storage/user/file", handler)
+
+    srv = await aiohttp_server(app)
+
+    async with make_client(srv.make_url("/")) as client:
+        buf = bytearray()
+        async for chunk in client.storage.open(URL("storage:file"), 0):
+            buf.extend(chunk)
+        assert buf == b"01234"
+
+        with pytest.raises(RuntimeError):
+            async for chunk in client.storage.open(URL("storage:file"), 5):
+                pass
+
+
 async def test_storage_open_directory(
     aiohttp_server: _TestServerFactory, make_client: _MakeClient
 ) -> None:
@@ -868,7 +991,10 @@ async def test_storage_upload_dir_doesnt_exist(make_client: _MakeClient) -> None
 
 
 async def test_storage_upload_not_a_file(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = Path(os.devnull).absolute()
     target_path = storage_path / "file.txt"
@@ -890,7 +1016,10 @@ async def test_storage_upload_not_a_file(
 
 
 async def test_storage_upload_regular_file_to_existing_file_target(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
     file_size = file_path.stat().st_size
@@ -916,7 +1045,10 @@ async def test_storage_upload_regular_file_to_existing_file_target(
 
 
 async def test_storage_upload_regular_file_to_existing_dir(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
     folder = storage_path / "folder"
@@ -930,7 +1062,10 @@ async def test_storage_upload_regular_file_to_existing_dir(
 
 
 async def test_storage_upload_regular_file_to_existing_file(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
     folder = storage_path / "folder"
@@ -949,7 +1084,10 @@ async def test_storage_upload_regular_file_to_existing_file(
 
 
 async def test_storage_upload_regular_file_to_existing_dir_with_trailing_slash(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
     folder = storage_path / "folder"
@@ -963,7 +1101,10 @@ async def test_storage_upload_regular_file_to_existing_dir_with_trailing_slash(
 
 
 async def test_storage_upload_regular_file_to_existing_non_dir(
-    storage_server: Any, make_client: _MakeClient, storage_path: Path
+    storage_server: Any,
+    make_client: _MakeClient,
+    storage_path: Path,
+    small_block_size: None,
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
     path = storage_path / "file"
@@ -977,7 +1118,7 @@ async def test_storage_upload_regular_file_to_existing_non_dir(
 
 
 async def test_storage_upload_regular_file_to_not_existing(
-    storage_server: Any, make_client: _MakeClient
+    storage_server: Any, make_client: _MakeClient, small_block_size: None
 ) -> None:
     file_path = DATA_FOLDER / "file.txt"
 
@@ -1223,31 +1364,78 @@ async def test_storage_upload_file_update(
     tmp_path: Path,
     storage_path: Path,
     zero_time_threshold: None,
+    small_block_size: None,
 ) -> None:
     storage_file = storage_path / "file.txt"
     local_file = tmp_path / "file.txt"
+    src = URL(local_file.as_uri())
+    dst = URL("storage:file.txt")
 
-    local_file.write_bytes(b"old")
+    # No destination file
+    assert not storage_file.exists()
+    local_file.write_bytes(b"old content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_file(
-            URL(local_file.as_uri()), URL("storage:file.txt"), update=True
-        )
+        await client.storage.upload_file(src, dst, update=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, update=True)
+    assert storage_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    storage_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, update=True)
     assert storage_file.read_bytes() == b"old"
 
-    local_file.write_bytes(b"new")
-    async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_file(
-            URL(local_file.as_uri()), URL("storage:file.txt"), update=True
-        )
-    assert storage_file.read_bytes() == b"new"
 
-    await asyncio.sleep(5)
-    storage_file.write_bytes(b"xxx")
+async def test_storage_upload_file_continue(
+    storage_server: Any,
+    make_client: _MakeClient,
+    tmp_path: Path,
+    storage_path: Path,
+    zero_time_threshold: None,
+    small_block_size: None,
+) -> None:
+    storage_file = storage_path / "file.txt"
+    local_file = tmp_path / "file.txt"
+    src = URL(local_file.as_uri())
+    dst = URL("storage:file.txt")
+
+    # No destination file
+    assert not storage_file.exists()
+    local_file.write_bytes(b"content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_file(
-            URL(local_file.as_uri()), URL("storage:file.txt"), update=True
-        )
-    assert storage_file.read_bytes() == b"xxx"
+        await client.storage.upload_file(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    storage_file.write_bytes(b"old content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    storage_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    storage_file.write_bytes(b"old long content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_file(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"new content"
 
 
 async def test_storage_upload_dir_update(
@@ -1261,28 +1449,76 @@ async def test_storage_upload_dir_update(
     local_dir = tmp_path / "folder"
     local_file = local_dir / "nested" / "file.txt"
     local_file.parent.mkdir(parents=True)
+    src = URL(local_dir.as_uri())
+    dst = URL("storage:folder")
 
-    local_file.write_bytes(b"old")
+    # No destination file
+    assert not storage_file.exists()
+    local_file.write_bytes(b"old content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_dir(
-            URL(local_dir.as_uri()), URL("storage:folder"), update=True
-        )
+        await client.storage.upload_dir(src, dst, update=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, update=True)
+    assert storage_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    storage_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, update=True)
     assert storage_file.read_bytes() == b"old"
 
-    local_file.write_bytes(b"new")
-    async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_dir(
-            URL(local_dir.as_uri()), URL("storage:folder"), update=True
-        )
-    assert storage_file.read_bytes() == b"new"
 
-    await asyncio.sleep(5)
-    storage_file.write_bytes(b"xxx")
+async def test_storage_upload_dir_continue(
+    storage_server: Any,
+    make_client: _MakeClient,
+    tmp_path: Path,
+    storage_path: Path,
+    zero_time_threshold: None,
+    small_block_size: None,
+) -> None:
+    storage_file = storage_path / "folder" / "nested" / "file.txt"
+    local_dir = tmp_path / "folder"
+    local_file = local_dir / "nested" / "file.txt"
+    local_file.parent.mkdir(parents=True)
+    src = URL(local_dir.as_uri())
+    dst = URL("storage:folder")
+
+    # No destination file
+    assert not storage_file.exists()
+    local_file.write_bytes(b"content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.upload_dir(
-            URL(local_dir.as_uri()), URL("storage:folder"), update=True
-        )
-    assert storage_file.read_bytes() == b"xxx"
+        await client.storage.upload_dir(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"content"
+
+    # Source file is newer
+    local_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(5)
+    storage_file.write_bytes(b"old content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    storage_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    storage_file.write_bytes(b"old long content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.upload_dir(src, dst, continue_=True)
+    assert storage_file.read_bytes() == b"new content"
 
 
 async def test_storage_download_file_update(
@@ -1294,28 +1530,74 @@ async def test_storage_download_file_update(
 ) -> None:
     storage_file = storage_path / "file.txt"
     local_file = tmp_path / "file.txt"
+    src = URL("storage:file.txt")
+    dst = URL(local_file.as_uri())
 
-    storage_file.write_bytes(b"old")
+    # No destination file
+    assert not local_file.exists()
+    storage_file.write_bytes(b"old content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_file(
-            URL("storage:file.txt"), URL(local_file.as_uri()), update=True
-        )
+        await client.storage.download_file(src, dst, update=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    storage_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, update=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, update=True)
     assert local_file.read_bytes() == b"old"
 
-    storage_file.write_bytes(b"new")
-    async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_file(
-            URL("storage:file.txt"), URL(local_file.as_uri()), update=True
-        )
-    assert local_file.read_bytes() == b"new"
 
-    await asyncio.sleep(2)
-    local_file.write_bytes(b"xxx")
+async def test_storage_download_file_continue(
+    storage_server: Any,
+    make_client: _MakeClient,
+    tmp_path: Path,
+    storage_path: Path,
+    zero_time_threshold: None,
+    small_block_size: None,
+) -> None:
+    storage_file = storage_path / "file.txt"
+    local_file = tmp_path / "file.txt"
+    src = URL("storage:file.txt")
+    dst = URL(local_file.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    storage_file.write_bytes(b"content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_file(
-            URL("storage:file.txt"), URL(local_file.as_uri()), update=True
-        )
-    assert local_file.read_bytes() == b"xxx"
+        await client.storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"content"
+
+    # Source file is newer
+    storage_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    local_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    local_file.write_bytes(b"old long content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_file(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
 
 
 async def test_storage_download_dir_update(
@@ -1329,28 +1611,75 @@ async def test_storage_download_dir_update(
     local_dir = tmp_path / "folder"
     local_file = local_dir / "nested" / "file.txt"
     storage_file.parent.mkdir(parents=True)
+    src = URL("storage:folder")
+    dst = URL(local_dir.as_uri())
 
-    storage_file.write_bytes(b"old")
+    # No destination file
+    assert not local_file.exists()
+    storage_file.write_bytes(b"old content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_dir(
-            URL("storage:folder"), URL(local_dir.as_uri()), update=True
-        )
+        await client.storage.download_dir(src, dst, update=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Source file is newer
+    storage_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, update=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, update=True)
     assert local_file.read_bytes() == b"old"
 
-    storage_file.write_bytes(b"new")
-    async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_dir(
-            URL("storage:folder"), URL(local_dir.as_uri()), update=True
-        )
-    assert local_file.read_bytes() == b"new"
 
-    await asyncio.sleep(2)
-    local_file.write_bytes(b"xxx")
+async def test_storage_download_dir_continue(
+    storage_server: Any,
+    make_client: _MakeClient,
+    tmp_path: Path,
+    storage_path: Path,
+    zero_time_threshold: None,
+) -> None:
+    storage_file = storage_path / "folder" / "nested" / "file.txt"
+    local_dir = tmp_path / "folder"
+    local_file = local_dir / "nested" / "file.txt"
+    storage_file.parent.mkdir(parents=True)
+    src = URL("storage:folder")
+    dst = URL(local_dir.as_uri())
+
+    # No destination file
+    assert not local_file.exists()
+    storage_file.write_bytes(b"content")
     async with make_client(storage_server.make_url("/")) as client:
-        await client.storage.download_dir(
-            URL("storage:folder"), URL(local_dir.as_uri()), update=True
-        )
-    assert local_file.read_bytes() == b"xxx"
+        await client.storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"content"
+
+    # Source file is newer
+    storage_file.write_bytes(b"new content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
+
+    # Destination file is newer, same size
+    await asyncio.sleep(2)
+    local_file.write_bytes(b"old content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is shorter
+    local_file.write_bytes(b"old")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"old content"
+
+    # Destination file is longer
+    local_file.write_bytes(b"old long content")
+    async with make_client(storage_server.make_url("/")) as client:
+        await client.storage.download_dir(src, dst, continue_=True)
+    assert local_file.read_bytes() == b"new content"
 
 
 async def test_storage_upload_dir_with_ignore_file_names(
