@@ -53,7 +53,13 @@ from neuromation.api import (
     get as api_get,
     login_with_token,
 )
-from neuromation.api.utils import asynccontextmanager
+
+
+if sys.version_info >= (3, 7):  # pragma: no cover
+    from contextlib import asynccontextmanager
+else:
+    from async_generator import asynccontextmanager
+
 from neuromation.cli.asyncio_utils import run
 from neuromation.cli.utils import resolve_job
 from tests.e2e.utils import FILE_SIZE_B, NGINX_IMAGE_NAME, JobWaitStateStopReached
@@ -187,13 +193,7 @@ class Helper:
             return await resolve_job(
                 job_name,
                 client=client,
-                status={
-                    JobStatus.PENDING,
-                    JobStatus.RUNNING,
-                    JobStatus.SUCCEEDED,
-                    JobStatus.CANCELLED,
-                    JobStatus.FAILED,
-                },
+                status=JobStatus.items(),
             )
 
     @run_async
@@ -620,14 +620,14 @@ class Helper:
         __tracebackhide__ = True
         async with api_get(timeout=CLIENT_TIMEOUT, path=self._nmrc_path) as client:
             id = await resolve_job(
-                id_or_name, client=client, status={JobStatus.PENDING, JobStatus.RUNNING}
+                id_or_name, client=client, status=JobStatus.active_items()
             )
             with suppress(ResourceNotFound, IllegalArgumentError):
                 await client.jobs.kill(id)
                 if wait:
                     while True:
                         stat = await client.jobs.status(id)
-                        if stat.status not in (JobStatus.PENDING, JobStatus.RUNNING):
+                        if stat.status.is_finished:
                             break
 
     kill_job = run_async(akill_job)
